# Game Raccoon directory_cache_read.py
# (c) Mathew Carr July 2020
#
# Given a binary file containing the contents of the Game Raccoon Flash
# in the range $200000-$3FFFFF, display the contents of the directory
# cache in a friendly manner.
#
# Copyright (c) 2020 Mathew Carr
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License Version 3
# with Additional Terms as found in the file
# LICENSE_AGPL_V3_ADDITIONAL_TERMS.txt.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# Usage:
# python directory_cache_read.py 
#
# The input file is expected to be named 'directory_cache_emulator.bin',
# and to be a complete appended Game Raccoon boot image either generated
# offline using the tools, or read from the BOOT partition of real Game Raccoon
# hardware and written to SD card to test if the PIC is generating the
# directory record cache correctly.

DIRECTORY_CACHE_BASE                      = 0x200000
DIRECTORY_CACHE_ENTRY_SIZE_BYTES          =    0x100       
DIRECTORY_CACHE_ENTRY_SIZE_WORDS          = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES/2)
DIRECTORY_CACHE_OFFSET_TO_DIRECTORY_COUNT = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(4*2))
DIRECTORY_CACHE_OFFSET_TO_FILE_COUNT      = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(3*2))
DIRECTORY_CACHE_OFFSET_TO_CONTENTS_BEGIN  = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(2*2))
DIRECTORY_CACHE_OFFSET_TO_PARENT_INDEX    = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(1*2))

def chunks(l, n):
   """Yield successive n-sized chunks from l."""
   for i in range(0, len(l), n):
      yield l[i:i + n]   

def toval(byte_pair):      
   return byte_pair[0] * 256 + byte_pair[1]
      
directory_cache_data = open("directory_cache_emulator.bin","rb").read()

for i,entry in enumerate(chunks(directory_cache_data, 256)):
   filename_ascii = entry[:entry.index(b'\x00')].decode('ascii')
   
   dir_count      = toval(entry[DIRECTORY_CACHE_OFFSET_TO_DIRECTORY_COUNT:][:2])
   file_count     = toval(entry[DIRECTORY_CACHE_OFFSET_TO_FILE_COUNT     :][:2])
   contents_begin = toval(entry[DIRECTORY_CACHE_OFFSET_TO_CONTENTS_BEGIN :][:2])
   parent_index   = toval(entry[DIRECTORY_CACHE_OFFSET_TO_PARENT_INDEX   :][:2])
   
   print("%3d: (%3d,%3d,%3d,%3d) %s " % (i,dir_count,file_count,contents_begin,parent_index,filename_ascii))