; Game Raccoon entry_point.68k.asm
; Game Raccoon Revision 1 68000 Menu Firmware
; (c) Mathew Carr July 2020
;
; This file contains the main menu firmware of the Game Raccoon cartridge.
; It contains the Mega Drive header and some startup things.
; Almost all the binary resources (images, basically) are included into
; this file.
; Almost all the interactive code is contained in the RAM-resident part.
; Any time the cart is isolated, routines in this file will not be available!
;
; Copyright (c) 2020 Mathew Carr
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU Affero General Public License Version 3
; with Additional Terms as found in the file
; LICENSE_AGPL_V3_ADDITIONAL_TERMS.txt.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU Affero General Public License for more details.
; 
; You should have received a copy of the GNU Affero General Public License
; along with this program.  If not, see <https://www.gnu.org/licenses/>.
;
; Use of the Game Raccoon logotype, name, characters and other identifying
; material is subject to license conditions.
;
; Revision 0 started:
; 25th December 2019.
;
; Revision 1 begins with aplomb:
; 27th April 2020.

        include "megadrive_defines.68k.asm"

; *** Game Raccoon does not support interrupts of any kind. ***
; The exception vector table is fixed at the start of the address
; space on Motorola 68000, so when isolation is in effect, this table
; will be unreachable.
unhandled_exception                         = $00000000
vblank                                      = $00000000
hblank                                      = $00000000

VECTOR_BUS_ERROR                            = unhandled_exception
VECTOR_ADDRESS_ERROR                        = unhandled_exception
VECTOR_ILLEGAL_INSTRUCTION                  = unhandled_exception
VECTOR_DIVISION_BY_ZERO                     = unhandled_exception
VECTOR_CHK_EXCEPTION                        = unhandled_exception
VECTOR_TRAPV_EXCEPTION                      = unhandled_exception
VECTOR_PRIVILEGE_VIOLATION_EXCEPTION        = unhandled_exception
VECTOR_TRACE_EXCEPTION                      = unhandled_exception
VECTOR_LINE_1010_EMULATOR                   = unhandled_exception
VECTOR_LINE_1111_EMULATOR                   = unhandled_exception
VECTOR_RESERVED_EXCEPTION                   = unhandled_exception
VECTOR_SPURIOUS_EXCEPTION                   = unhandled_exception
VECTOR_INTERRUPT_REQUEST_LEVEL_1            = unhandled_exception
VECTOR_INTERRUPT_REQUEST_LEVEL_2            = unhandled_exception
VECTOR_INTERRUPT_REQUEST_LEVEL_3            = unhandled_exception
VECTOR_INTERRUPT_REQUEST_LEVEL_4_VDP_HBLANK = hblank
VECTOR_INTERRUPT_REQUEST_LEVEL_5            = unhandled_exception
VECTOR_INTERRUPT_REQUEST_LEVEL_6_VDP_VBLANK = vblank
VECTOR_INTERRUPT_REQUEST_LEVEL_7            = unhandled_exception
VECTOR_TRAP_00_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_01_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_02_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_03_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_04_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_05_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_06_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_07_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_08_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_09_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_10_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_11_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_12_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_13_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_14_EXCEPTION                    = unhandled_exception
VECTOR_TRAP_15_EXCEPTION                    = unhandled_exception

STACK_POINTER_START = RAM_END    ; Full descending stack points to address beyond end of RAM.

        ; ROM Header begins at $00:        
        org $00
        dc.l STACK_POINTER_START
        dc.l main                                              ; Normally I'd use the symbol CART_ENTRY_POINT here, but asmx prevents this, unlike vasm.        
        ; Bytes $08 to $FF are the Motorola 68000 exception vector table.
        dc.l VECTOR_BUS_ERROR                                  ; If you need to find the value of a symbol, just put it here in the vector table
        dc.l VECTOR_ADDRESS_ERROR                              ; we're not using these values for anything, so you can just use these to put
                                                               ; values where you can see them easily in a hex editor if you like.
        dc.l VECTOR_ILLEGAL_INSTRUCTION
        dc.l VECTOR_DIVISION_BY_ZERO
        dc.l VECTOR_CHK_EXCEPTION
        dc.l VECTOR_TRAPV_EXCEPTION

        dc.l VECTOR_PRIVILEGE_VIOLATION_EXCEPTION
        dc.l VECTOR_TRACE_EXCEPTION
        dc.l VECTOR_LINE_1010_EMULATOR
        dc.l VECTOR_LINE_1111_EMULATOR

        dc.l unhandled_exception                              ; Reserved interrupt vector entries that aren't implemented in 68000.
        dc.l unhandled_exception
        dc.l unhandled_exception
        dc.l unhandled_exception

        dc.l unhandled_exception
        dc.l unhandled_exception
        dc.l unhandled_exception
        dc.l unhandled_exception

        dc.l unhandled_exception
        dc.l unhandled_exception
        dc.l unhandled_exception
        dc.l unhandled_exception

        dc.l VECTOR_SPURIOUS_EXCEPTION
        dc.l VECTOR_INTERRUPT_REQUEST_LEVEL_1
        dc.l VECTOR_INTERRUPT_REQUEST_LEVEL_2
        dc.l VECTOR_INTERRUPT_REQUEST_LEVEL_3

        dc.l VECTOR_INTERRUPT_REQUEST_LEVEL_4_VDP_HBLANK
        dc.l VECTOR_INTERRUPT_REQUEST_LEVEL_5
        dc.l VECTOR_INTERRUPT_REQUEST_LEVEL_6_VDP_VBLANK
        dc.l VECTOR_INTERRUPT_REQUEST_LEVEL_7

        dc.l VECTOR_TRAP_00_EXCEPTION
        dc.l VECTOR_TRAP_01_EXCEPTION
        dc.l VECTOR_TRAP_02_EXCEPTION
        dc.l VECTOR_TRAP_03_EXCEPTION

        dc.l VECTOR_TRAP_04_EXCEPTION
        dc.l VECTOR_TRAP_05_EXCEPTION
        dc.l VECTOR_TRAP_06_EXCEPTION
        dc.l VECTOR_TRAP_07_EXCEPTION

        dc.l VECTOR_TRAP_08_EXCEPTION
        dc.l VECTOR_TRAP_09_EXCEPTION
        dc.l VECTOR_TRAP_10_EXCEPTION
        dc.l VECTOR_TRAP_11_EXCEPTION

        dc.l VECTOR_TRAP_12_EXCEPTION
        dc.l VECTOR_TRAP_13_EXCEPTION
        dc.l VECTOR_TRAP_14_EXCEPTION
        dc.l VECTOR_TRAP_15_EXCEPTION
        
        ; All other exceptions are FPU or MMU related and unhandled.
        ; vasm fills with $00, asmx fills with $FF.
        ; I could use this ROM space for things if I want.

        ; SEGA Mega Drive header begins at $100:
        org $100
        dc.b "SEGA MEGA DRIVE "                                 ; 16 - Console name
        org $110
        dc.b "(C)MATT 2020.JUL"                                 ; 16 - Release date "(C)comp year.mon" (TODO: Update this appropriately automatically?)
        org $120
        dc.b "GAME RACCOON BOOT IMAGE MATHEW CARR             " ; 48 - Domestic name
        org $150
        dc.b "GAME RACCOON BOOT IMAGE MATHEW CARR             " ; 48 - International name
        org $180
        dc.b "GR GRACCOON-01"                                   ; 14 - Version string
        org $18E
        dc.w $1234                                              ;  2 - Checksum (TODO: It's not important, it boots on my PAL Model 1, but...)
        org $190
        dc.b "J               "                                 ; 16 - Device support. J is joypad. Don't support lightguns, modems, etc.
        org $1A0
        dc.l $00000000                                          ;  4 - ROM start
        dc.l $0000FFFF                                          ;  4 - ROM end (TODO: Magically change this to be ROM size, also padded to nearest 64kbyte?)
        org $1A8
        dc.l RAM_START                                          ;  4 - RAM start
        dc.l RAM_LAST_BYTE                                      ;  4 - RAM end
        org $1B0
        dc.b "            "                                     ; 12 - Non-volatile memory information area
        org $1BC
        dc.b "            "                                     ; 12 - Modem support
        org $1C8
        dc.b "          "                                       ; 40 - Reserved
        dc.b "          "
        dc.b "          "
        dc.b "          "
        org $1F0
        dc.b "JUE"                                              ;  3 - Region support.
        org $1F3
        dc.b "             "                                    ; 13 - Reserved
; Header ends.

; RAM memory map assignments for Game Raccoon boot ROM:

; The RAM-resident Game Raccoon executable image is installed
; at the front of RAM.
GAME_RACCOON_RAM_RESIDENT_DESTINATION = RAM_START

        org $200
; We're going to put the binaries blob at $200 so we can calculate all the offsets
; and lengths in advance. 'main' will be pushed down into the ROM by the binaries
; archive, but that is of no consequence since the Mega Drive cart header is
; generated from the 'main' symbol.
        
        align   4 ; Note that asmx align is the byte count, not the exponent of 2.
; Entry point when cart is booted.
main:
        ; Initialise Mega Drive:
        ; We want to satisfy the TMSS check as soon as possible.
        interrupts_disable
        move.b  (MM_VERSION_REGISTER+1),d0       ; Load the low byte from the version register.
        and.b   #$0F,d0
        beq.s   tmss_satisfy_tmss_unnecessary
        move.l  #'SEGA',(MM_TMSS_REGISTER)       ; Satisy TMSS on version != zero consoles by writing SEGA longword to its register.
tmss_satisfy_tmss_unnecessary:                   ; Branch here if we're on a version zero console that does not require TMSS compliance.
        ; asmx uses single quotes for series-of-ASCII-chars literal, vasm uses double quotes.
        
        ; Past this point, the console is entirely at our command.
                
        ; I'm going to copy the Game Raccoon RAM-Resident block
        ; into RAM at $FF0000...
        lea     game_raccoon_ram_resident_rom_included_block_start,a0
        lea     GAME_RACCOON_RAM_RESIDENT_DESTINATION,a1
        move.w  #(GAME_RACCOON_RAM_RESIDENT_ROM_BLOCK_LENGTH_LONGS-1),d0
game_raccoon_ram_resident_copy_next_long:
        move.l  (a0)+,(a1)+
        dbf     d0,game_raccoon_ram_resident_copy_next_long
        
        ; Okay, now we can perform useful things.
        
        ; First, let's read the controller and see if we
        ; should activate emergency quick boot mode.
        ; I should be able to call into RAM-resident
        ; routines since I included the source not the 
        ; binary of the RAM-resident part this time.
        lea     pad_state,a0
        jsr     controllers_reset
        lea     pad_state,a0
        jsr     controllers_read_and_derive
        
        btst    #CONTROLLER_S_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     game_raccoon_start_normal                                             ; Need to jump like this as conditional branches have to be short.
        jmp     game_raccoon_start_emergency
        
        ; Normal Game Raccoon start.        
game_raccoon_start_normal:  
        jsr     vdp_clear_all_cram                                                    ; Palettes to zero first.
        jsr     vdp_initialise_registers     

        jsr     z80_initialise
        
        jsr     vdp_clear_all_vram                                                    ; Clears the sprite table too, which is held in VRAM of course.
        jsr     vdp_clear_all_vsram                                                   ; Clears the VSRAM.
        
        jsr     psg_fm_initialise                                                     ; Silence all output. 
        
        jsr     palette_fade_struct_initialise
        
        ; It's time to start loading up some resources into VRAM.
        ; I think I can cram all the backgrounds, fonts and raccoon sprites
        ; into VRAM simultaneously, so do it here.
        jsr     intro_screen_load_resources_populate_nametables
        
        jmp     intro_screen_begin
        
; intro_screen_load_resources_populate_nametables:        
; Invalidates all registers.
intro_screen_load_resources_populate_nametables:
        ; Load the texture tiles into 1-32
        ; 32 tiles * $20 bytes = 1024 bytes = 512 words.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$0000+(PATTERN_SIZE_BYTES*1)
        move.w  #(512-1),d0
        lea     menutexture_img_bin,a0
intro_screen_load_resources_populate_nametables_load_tiles_next_tile:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,intro_screen_load_resources_populate_nametables_load_tiles_next_tile
        
        ; Write the standard pattern to the full 512x256 nametable of Plane B.
        ; I just store the tegels in ROM because I have a lot of it!
        ; 512x256 plane = 64x32 = 2048 tegels
        vram_write VDP_LCOMMAND_VRAM_WRITE,$E000
        move.w  #(2048-1),d0
        lea     intro_screen_plane_b_texture_tegels,a0
intro_screen_load_resources_populate_nametables_load_plane_b_tegels_next_tegel:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,intro_screen_load_resources_populate_nametables_load_plane_b_tegels_next_tegel
        
        ; Load the Game Raccoon intro graphic into tiles 33-7392
        ; 230 tiles * $20 bytes = 7360 bytes = 3680 words.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$0000+(PATTERN_SIZE_BYTES*33)
        move.w  #(intro_game_raccoon_logo_image_bin_length_words-1),d0
        lea     intro_game_raccoon_logo_image_bin,a0
intro_screen_load_resources_populate_nametables_load_game_raccoon_logo_image_tiles_next_tile:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,intro_screen_load_resources_populate_nametables_load_game_raccoon_logo_image_tiles_next_tile
        
        ; Load the Game Raccoon intro graphic tegels into Plane A.
GAME_RACCOON_INTRO_GRAPHIC_TILES_W = 23
GAME_RACCOON_INTRO_GRAPHIC_TILES_H = 10

        move.w  #(GAME_RACCOON_INTRO_GRAPHIC_TILES_H-1),d7
        vram_write_dx VDP_LCOMMAND_VRAM_WRITE,($C000+(2*(9+64*3))),d0                 ; d0 contains VDP command with write address in. It's autoincremented, but we need to jump at the end of a line.
        move.w  #33+VDP_PALETTE_1,d1                                                  ; d1 contains tegel to write.
        
intro_screen_load_resources_intro_graphic_tegels_next_row:
        move.l  d0,(MM_VDP_CONTROL)
        
        move.w  #(GAME_RACCOON_INTRO_GRAPHIC_TILES_W-1),d6
intro_screen_load_resources_intro_graphic_tegels_next_tile:
        move.w  d1,(MM_VDP_DATA)
        add.w   #1,d1
        dbf     d6,intro_screen_load_resources_intro_graphic_tegels_next_tile
        
        add.l   #((2*(64))<<16),d0                                                    ; Advance to the next line by adding width of Plane to d0, not forgetting swizzle.
        dbf     d7,intro_screen_load_resources_intro_graphic_tegels_next_row
        ; Loading intro graphic into Plane A done.
        
        ; Write 'Game Development Cartridge' below logo.
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #7,d1                                                                 ; d1.l: X
        move.l  #13,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_intro_game_raccoon_game_development_cartridge,a0               ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8;16
        
        ; Load the Saikyo/Torus 8x8 font into $B420 - $BFFF (tile base decimal 1441) directly before the $C000 plane tegels.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$B420
        move.w  #(saikyo_torus_8x8_font_image_bin_length_words-1),d0
        lea     saikyo_torus_8x8_font_image_bin,a0
intro_screen_load_resources_populate_nametables_load_saikyo_tiles_next_tile:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,intro_screen_load_resources_populate_nametables_load_saikyo_tiles_next_tile
        
        ; Load the GSMPlay 8x16 font into $9CA0 - $B41F (tile base decimal 1253) directly before the above font.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$9CA0
        move.w  #(tepples_gsmplay_8x16_font_image_bin_length_words-1),d0
        lea     tepples_gsmplay_8x16_font_image_bin,a0
intro_screen_load_resources_populate_nametables_load_tepples_tiles_next_tile:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,intro_screen_load_resources_populate_nametables_load_tepples_tiles_next_tile
                        
        ; Load the button sprites into $94A0 - $9C9F (tile base decimal 1189)
        vram_write VDP_LCOMMAND_VRAM_WRITE,$94A0
        move.w  #(sprites_buttons_abcs_image_bin_length_words-1),d0
        lea     sprites_buttons_abcs_image_bin,a0
intro_screen_load_resources_populate_nametables_load_button_sprites_next_tile:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,intro_screen_load_resources_populate_nametables_load_button_sprites_next_tile
                        
        ; Load the raccoon selection hand sprites into $9320 - $949F (tile base decimal 1177)
        vram_write VDP_LCOMMAND_VRAM_WRITE,$9320
        move.w  #(sprites_raccoon_selection_hand_arrow_widgets_bin_length_words-1),d0
        lea     sprites_raccoon_selection_hand_arrow_widgets_bin,a0
intro_screen_load_resources_populate_nametables_load_raccoon_selection_hand_arrow_widgets_sprites_next_tile:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,intro_screen_load_resources_populate_nametables_load_raccoon_selection_hand_arrow_widgets_sprites_next_tile
        
        rts        
        
; plane_b_scrolling_background_initialise:
; Reset the plane_b state.
; No registers invalidated.                
plane_b_scrolling_background_initialise:
        move.l  #0,plane_b_scrolling_background_status_struct_scroll_counter
        move.l  #0,plane_b_scrolling_background_status_struct_scroll_speed
        rts                   
        
; palette_fade_struct_initialise:
; Sets all palette_fade_struct fields to zero.
; Invalidates all registers.
palette_fade_struct_initialise:
        lea     palette_fade_struct,a0
        move.l  #(palette_fade_struct_size_all_lines_words-1),d0
palette_fade_struct_initialise_next_word:        
        move.w  #0,(a0)+
        dbf     d0,palette_fade_struct_initialise_next_word        
        rts
        
; vdp_initialise_registers;
; Put the VDP into a good state. VBlank interrupts enabled. HBlank disabled. 320x224 resolution.
; Invalidates all registers.
vdp_initialise_registers:
        lea     (MM_VDP_CONTROL),a6
        move.w  #(VDP_REGISTER_MODE_REGISTER_1|$00),(a6)
        move.w  #(VDP_REGISTER_MODE_REGISTER_2|VDP_REGISTER_2_VERTICAL_INTERRUPT_ENABLE|VDP_REGISTER_2_DISPLAY_ENABLE|VDP_REGISTER_2_MEGA_DRIVE_VIDEO_MODE_ENABLE),(a6)
        move.w  #(VDP_REGISTER_MODE_REGISTER_3|VDP_REGISTER_3_VERTICAL_SCROLL_COMPLETELY|VDP_REGISTER_3_HORIZONTAL_SCROLL_COMPLETELY),(a6)
        move.w  #(VDP_REGISTER_MODE_REGISTER_4|VDP_REGISTER_4_320_PIXEL_WIDE_DISPLAY_ENABLE),(a6)

        move.w  #(VDP_REGISTER_AUTO_INCREMENT_VALUE|$02),(a6)                         ; We're going to assume this all over the place. If you change it away, CHANGE IT BACK.

        move.w  #(VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_C000),(a6)            
        move.w  #(VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_E000),(a6)          
        move.w  #(VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_D000),(a6)            
        move.w  #(VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_F400),(a6)      
        move.w  #(VDP_REGISTER_SPRITE_TABLE_LOCATION_F000),(a6)                  

        move.w  #(VDP_REGISTER_BACKGROUND_COLOUR|$00),(a6)
        move.w  #(VDP_REGISTER_PLANE_SIZE|$01),(a6)                                   ; A and B are both 64x 32y.

        move.w  #(VDP_REGISTER_WINDOW_PLANE_HORIZONTAL_EXTENT|$00),(a6)               ; Full zero prevents window plane display.
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|$00),(a6)

        move.w  #(VDP_REGISTER_UNUSED_SPRITE_PATTERN_GENERATOR_BASE_ADDRESS|$00),(a6)
        move.w  #(VDP_REGISTER_UNUSED_08|$00),(a6)
        move.w  #(VDP_REGISTER_UNUSED_09|$00),(a6)
        move.w  #(VDP_REGISTER_UNUSED_NAMETABLE_PATTERN_GENERATOR_BASE_ADDRESS|$00),(a6)
        rts

; z80_initialise:        
; Make the z80 inert for the time being.
; Invalidates all registers.
; According to the sega documentation, the correct sequence for loading the Z80 program is:
;    (1) BUS REQ ON
;    (2) BUS RESET OFF
;    (3) 68K copies program into Z-80 S-RAM
;    (4) BUS RESET ON
;    (5) BUS REQ OFF
;    (6) BUS RESET OFF
z80_initialise:
        z80_bus_request_engage
        z80_reset_disengage
        z80_wait_bus_available

        lea     MM_Z80_MEMORY_SPACE,a0

        move.b  #$C3,(a0)+                         ; Put a [jp $0000] at $0000.
        move.b  #$00,(a0)+
        move.b  #$00,(a0)

        z80_reset_engage
        z80_bus_request_disengage
        z80_reset_disengage
        rts
        
; psg_fm_initialise:
; Invalidates all registers.        
; The sound chips are on the Z80 bus, so we request it as above
; then send the values we need.
psg_fm_initialise:
        z80_bus_request_engage
        z80_wait_bus_available

        ; Silence the SN76489, by setting attenuation of all channels to F (full attenuation = silence).
        lea     MM_SN76489_PSG,a0
        move.b  #(PSG_LATCH_BIT|PSG_CHANNEL_0|PSG_TYPE_VOLUME_DATA|$F),(a0)  
        nop        
        move.b  #(PSG_LATCH_BIT|PSG_CHANNEL_1|PSG_TYPE_VOLUME_DATA|$F),(a0)  
        nop       
        move.b  #(PSG_LATCH_BIT|PSG_CHANNEL_2|PSG_TYPE_VOLUME_DATA|$F),(a0)         
        nop
        move.b  #(PSG_LATCH_BIT|PSG_CHANNEL_3|PSG_TYPE_VOLUME_DATA|$F),(a0)  
        
        ; Silence the YM2612:
        lea     MM_YM2612_ADDRESS_P1,a0
        lea     MM_YM2612_DATA_P1,a1
        ; For each of the six channels (lower three bits), note off all four operators (upper four bits).
        YM2612_wait
        move.b  #YM2612_28_NOTE_ON_CONTROL,(a0)
        YM2612_wait
        move.b  #%00000000|YM2612_28_CHANNEL_0,(a1)
        YM2612_wait
        move.b  #%00000000|YM2612_28_CHANNEL_1,(a1)
        YM2612_wait
        move.b  #%00000000|YM2612_28_CHANNEL_2,(a1)
        YM2612_wait
        move.b  #%00000000|YM2612_28_CHANNEL_3,(a1)
        YM2612_wait
        move.b  #%00000000|YM2612_28_CHANNEL_4,(a1)
        YM2612_wait
        move.b  #%00000000|YM2612_28_CHANNEL_5,(a1)
        ; For each of the three channels in each of the two parts, set
        ; the TL of each operator within each channel to 127.
        move.w  #(2-1),d0                                                             ; Two parts.
psg_fm_initialise_next_part:
        move.b  #$40,d1                                                               ; 40h is the data slot for the TLs of all operators across all channels.
        move.w  #(16-1),d2                                                            ; 16 entries across TL struct.
psg_fm_initialise_next_targeted_data_slot:
        YM2612_wait
        move.b  d1,(a0)                                                               ; Write to ADDRESS_P* the data slot we're targeting.
        addi.b  #1,d1                                                                 ; Advance targeted data slot to next across the TL row.
        YM2612_wait
        move.b  #127,(a1)                                                             ; Write to DATA_P* the maximum TL (minimum volume) value to the current slot.
        dbf     d2,psg_fm_initialise_next_targeted_data_slot        
        lea     MM_YM2612_ADDRESS_P2,a0                                               ; Swap to the other Part.
        lea     MM_YM2612_DATA_P2,a1
        dbf     d0,psg_fm_initialise_next_part
                
        z80_bus_request_disengage
        rts        
        
; End of ROM-resident code.        
;----------------------------------------------

        align   4
        include "included_binaries.68k.asm"   
        
; End of ROM-resident data and code.        
;----------------------------------------------
        
        ; Include the RAM-resident portion of the code contiguous in ROM,
        ; but with instructions based at the corresponding position in RAM.
        ; This file must always be a multiple of 4 large.
        ; It expects to be assembled at a multiple of 4.
        align   4
game_raccoon_ram_resident_rom_included_block_start:        
        include "raccoonramresident.68k.asm"