; Game Raccoon megadrive_defines.68k.asm
; Game Raccoon Revision 1 68000 Menu Firmware
; (c) Mathew Carr July 2020
;
; Various defines used throughout the Mega Drive-side of the code.
;
; Copyright (c) 2020 Mathew Carr
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU Affero General Public License Version 3
; with Additional Terms as found in the file
; LICENSE_AGPL_V3_ADDITIONAL_TERMS.txt.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU Affero General Public License for more details.
; 
; You should have received a copy of the GNU Affero General Public License
; along with this program.  If not, see <https://www.gnu.org/licenses/>.

RAM_START                            = $00FF0000
RAM_END                              = $01000000
RAM_LAST_BYTE                        = (RAM_END-1)
RAM_SIZE_BYTES                       = $00010000
RAM_SIZE_WORDS                       = RAM_SIZE_BYTES/2
RAM_SIZE_LONGS                       = RAM_SIZE_BYTES/4

VRAM_SIZE_BYTES                      = $00010000
VRAM_SIZE_WORDS                      = VRAM_SIZE_BYTES/2
VRAM_SIZE_LONGS                      = VRAM_SIZE_BYTES/4

VSRAM_SIZE_WORDS                     = 40
VSRAM_SIZE_BYTES                     = VSRAM_SIZE_WORDS*2
VSRAM_SIZE_LONGS                     = VSRAM_SIZE_BYTES/4

CRAM_SIZE_WORDS                      = 16*4
CRAM_SIZE_BYTES                      = CRAM_SIZE_WORDS*2

PATTERN_SIZE_BYTES                   = ((8*8*4)/8)            ; A pattern is an 8x8 tile, 4 bits per pixel depth.
PATTERN_SIZE_WORDS                   = PATTERN_SIZE_BYTES/2

;;;

; $000.l = address to set stack on game boot. VALUE MUST BE EVEN!
; $100.l = "SEGA"
CART_STACK_POSITION                    = $000
CART_ENTRY_POINT                       = $004
CART_SEGA_TEXT_HEADER                  = $100

; Equates for the Mega Drive's memory mapped peripherals.
MM_Z80_MEMORY_SPACE                  = $00A00000

MM_VERSION_REGISTER                  = $00A10000 ; Read only word long.

MM_CONTROLLER_1_DATA                 = $00A10002
MM_CONTROLLER_2_DATA                 = $00A10004
MM_EXPANSION_PORT_DATA               = $00A10006

MM_CONTROLLER_1_CONTROL              = $00A10008
MM_CONTROLLER_2_CONTROL              = $00A1000A
MM_EXPANSION_PORT_CONTROL            = $00A1000C

; Use this when we're cycling through the pads and we want to nn(An) access the CONTROL part
; when An points to the data register.
MM_CONTROLLER_DATA_TO_CONTROL_OFFSET = MM_CONTROLLER_1_CONTROL-MM_CONTROLLER_1_DATA
MM_CONTROLLER_DATA_SOURCE_STRIDE     = MM_CONTROLLER_2_DATA-MM_CONTROLLER_1_DATA

MM_CONTROLLER_1_SERIAL_TX            = $00A1000E
MM_CONTROLLER_1_SERIAL_RX            = $00A10010
MM_CONTROLLER_1_SERIAL_CONTROL       = $00A10012
MM_CONTROLLER_2_SERIAL_TX            = $00A10014
MM_CONTROLLER_2_SERIAL_RX            = $00A10016
MM_CONTROLLER_2_SERIAL_CONTROL       = $00A10018
MM_EXPANSION_PORT_SERIAL_TX          = $00A1001A
MM_EXPANSION_PORT_SERIAL_RX          = $00A1001C
MM_EXPANSION_PORT_SERIAL_CONTROL     = $00A1001E

MM_MEMORY_MODE_REGISTER              = $00A11000

MM_Z80_BUS_REQUEST                   = $00A11100
MM_Z80_RESET                         = $00A11200

MM_TIME_ADDRESS_BEGIN                = $00A13000 ; Writes to this area will cause !TIME to be pulled low in cart, this hits the Game Raccoon TIME register.
MM_TIME_ADDRESS_END                  = $00A13100

MM_TMSS_REGISTER                     = $00A14000 ; Longword sized.

MM_VDP_DATA                          = $00C00000
MM_VDP_CONTROL                       = $00C00004
MM_VDP_HV_COUNTER                    = $00C00008

MM_SN76489_PSG                       = $00C00011 ; Write bytes only.

PSG_LATCH_BIT                        = %10000000
PSG_CHANNEL_0                        = %00000000
PSG_CHANNEL_1                        = %00100000
PSG_CHANNEL_2                        = %01000000
PSG_CHANNEL_3                        = %01100000
PSG_TYPE_VOLUME_DATA                 = %00010000
PSG_TYPE_TONE_NOISE_DATA             = %00000000

MM_YM2612_ADDRESS_P1                 = $00A04000 ; Write bytes only.
MM_YM2612_DATA_P1                    = $00A04001 ; P1 is used for modifying channels 1-3.
MM_YM2612_ADDRESS_P2                 = $00A04002 ; P2 is used for modifying channels 4-6.
MM_YM2612_DATA_P2                    = $00A04003

YM2612_STATUS_BYTE_BUSY_BIT          = 7

YM2612_22_LFO_FREQ                   = $22
YM2612_24_TIMER_A_MSB                = $24
YM2612_25_TIMER_A_LSB                = $25
YM2612_26_TIMER_B_MSB                = $26
YM2612_27_CH3_MODE_TIMER_ENABLES     = $27
YM2612_28_NOTE_ON_CONTROL            = $28
YM2612_2A_DAC_VALUE                  = $2A
YM2612_2B_DAC_ENABLE                 = $2B

YM2612_28_CHANNEL_0                  = %000
YM2612_28_CHANNEL_1                  = %001
YM2612_28_CHANNEL_2                  = %010
YM2612_28_CHANNEL_3                  = %100
YM2612_28_CHANNEL_4                  = %101
YM2612_28_CHANNEL_5                  = %110

; Constants with the write bit already set.
VDP_REGISTER_WRITE_COMMAND                                   = $8000
VDP_REGISTER_MODE_REGISTER_1                                 = VDP_REGISTER_WRITE_COMMAND|$0004 ; We're going to cheat and put the 1 that's expected here.
VDP_REGISTER_MODE_REGISTER_2                                 = VDP_REGISTER_WRITE_COMMAND|$0100
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION                     = VDP_REGISTER_WRITE_COMMAND|$0200
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION                      = VDP_REGISTER_WRITE_COMMAND|$0300
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION                     = VDP_REGISTER_WRITE_COMMAND|$0400
VDP_REGISTER_SPRITE_TABLE_LOCATION                           = VDP_REGISTER_WRITE_COMMAND|$0500
VDP_REGISTER_UNUSED_SPRITE_PATTERN_GENERATOR_BASE_ADDRESS    = VDP_REGISTER_WRITE_COMMAND|$0600
VDP_REGISTER_BACKGROUND_COLOUR                               = VDP_REGISTER_WRITE_COMMAND|$0700
VDP_REGISTER_UNUSED_08                                       = VDP_REGISTER_WRITE_COMMAND|$0800
VDP_REGISTER_UNUSED_09                                       = VDP_REGISTER_WRITE_COMMAND|$0900
VDP_REGISTER_HORIZONTAL_INTERRUPT_COUNTER                    = VDP_REGISTER_WRITE_COMMAND|$0A00
VDP_REGISTER_MODE_REGISTER_3                                 = VDP_REGISTER_WRITE_COMMAND|$0B00
VDP_REGISTER_MODE_REGISTER_4                                 = VDP_REGISTER_WRITE_COMMAND|$0C00
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION                 = VDP_REGISTER_WRITE_COMMAND|$0D00
VDP_REGISTER_UNUSED_NAMETABLE_PATTERN_GENERATOR_BASE_ADDRESS = VDP_REGISTER_WRITE_COMMAND|$0E00
VDP_REGISTER_AUTO_INCREMENT_VALUE                            = VDP_REGISTER_WRITE_COMMAND|$0F00
VDP_REGISTER_PLANE_SIZE                                      = VDP_REGISTER_WRITE_COMMAND|$1000
VDP_REGISTER_WINDOW_PLANE_HORIZONTAL_EXTENT                  = VDP_REGISTER_WRITE_COMMAND|$1100
VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT                    = VDP_REGISTER_WRITE_COMMAND|$1200
VDP_REGISTER_DMA_LENGTH_LOW_BYTE                             = VDP_REGISTER_WRITE_COMMAND|$1300
VDP_REGISTER_DMA_LENGTH_HIGH_BYTE                            = VDP_REGISTER_WRITE_COMMAND|$1400
VDP_REGISTER_DMA_SOURCE_ADDRESS_L                            = VDP_REGISTER_WRITE_COMMAND|$1500
VDP_REGISTER_DMA_SOURCE_ADDRESS_M                            = VDP_REGISTER_WRITE_COMMAND|$1600
VDP_REGISTER_DMA_SOURCE_ADDRESS_H                            = VDP_REGISTER_WRITE_COMMAND|$1700

VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_0000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00000000)
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_2000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00001000)
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_4000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00010000)
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_6000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00011000)
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_8000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00100000)
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_A000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00101000)
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_C000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00110000)
VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION_E000     = (VDP_REGISTER_PLANE_A_NAME_TABLE_LOCATION|%00111000)

VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_0000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00000000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_0800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00000010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_1000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00000100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_1800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00000110)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_2000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00001000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_2800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00001010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_3000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00001100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_3800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00001110)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_4000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00010000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_4800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00010010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_5000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00010100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_5800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00010110)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_6000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00011000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_6800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00011010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_7000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00011100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_7800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00011110)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_8000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00100000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_8800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00100010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_9000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00100100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_9800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00100110)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_A000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00101000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_A800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00101010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_B000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00101100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_B800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00101110)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_C000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00110000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_C800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00110010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_D000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00110100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_D800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00110110)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_E000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00111000)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_E800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00111010)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_F000      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00111100)
VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION_F800      = (VDP_REGISTER_WINDOW_NAME_TABLE_LOCATION|%00111110)

VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_0000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000000)
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_2000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000001)
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_4000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000010)
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_6000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000011)
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_8000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000100)
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_A000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000101)
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_C000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000110)
VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION_E000     = (VDP_REGISTER_PLANE_B_NAME_TABLE_LOCATION|%00000111)

VDP_REGISTER_SPRITE_TABLE_LOCATION_0000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_0200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_0400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_0600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_0800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_0A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_0C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_0E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00000111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_1E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00001111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_2E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00010111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_3E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00011111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_4E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00100111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_5E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00101111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_6E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00110111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_7E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%00111111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_8E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01000111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9A00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9C00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_9E00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01001111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_A000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_A200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_A400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_A600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_A800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_AA00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_AC00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_AE00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01010111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_B000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_B200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_B400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_B600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_B800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_BA00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_BC00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_BE00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01011111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_C000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_C200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_C400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_C600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_C800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_CA00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_CC00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_CE00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01100111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_D000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_D200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_D400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_D600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_D800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_DA00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_DC00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_DE00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01101111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_E000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_E200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_E400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_E600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_E800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_EA00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_EC00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_EE00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01110111)
VDP_REGISTER_SPRITE_TABLE_LOCATION_F000           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111000)
VDP_REGISTER_SPRITE_TABLE_LOCATION_F200           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111001)
VDP_REGISTER_SPRITE_TABLE_LOCATION_F400           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111010)
VDP_REGISTER_SPRITE_TABLE_LOCATION_F600           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111011)
VDP_REGISTER_SPRITE_TABLE_LOCATION_F800           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111100)
VDP_REGISTER_SPRITE_TABLE_LOCATION_FA00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111101)
VDP_REGISTER_SPRITE_TABLE_LOCATION_FC00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111110)
VDP_REGISTER_SPRITE_TABLE_LOCATION_FE00           = (VDP_REGISTER_SPRITE_TABLE_LOCATION|%01111111)

VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_0000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_0400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_0800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_0C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_1000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_1400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_1800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_1C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00000111)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_2000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_2400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_2800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_2C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_3000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_3400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_3800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_3C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00001111)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_4000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_4400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_4800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_4C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_5000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_5400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_5800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_5C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00010111)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_6000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_6400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_6800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_6C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_7000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_7400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_7800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_7C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00011111)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_8000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_8400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_8800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_8C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_9000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_9400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_9800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_9C00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00100111)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_A000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_A400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_A800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_AC00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_B000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_B400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_B800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_BC00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00101111)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_C000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_C400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_C800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_CC00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_D000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_D400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_D800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_DC00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00110111)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_E000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111000)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_E400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111001)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_E800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111010)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_EC00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111011)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_F000 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111100)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_F400 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111101)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_F800 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111110)
VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION_FC00 = (VDP_REGISTER_HORIZONTAL_SCROLL_DATA_LOCATION|%00111111)

VDP_REGISTER_1_EXTRA_VRAM_ENABLE               = %10000000 ; SEGA says always zero.
VDP_REGISTER_1_LEFTMOST_BLANK_ENABLE           = %00100000 ; SEGA says always zero.
VDP_REGISTER_1_HORIZONTAL_INTERRUPT_ENABLE     = %00010000
VDP_REGISTER_1_HV_COUNTER_LATCH_ENABLE         = %00000010
VDP_REGISTER_1_DISPLAY_ENABLE_AND_OVERLAY      = %00000001 ; SEGA says always zero.

VDP_REGISTER_2_TMS9918A_DISPLAY_ENABLE         = %10000000 ; SEGA says always zero.
VDP_REGISTER_2_DISPLAY_ENABLE                  = %01000000
VDP_REGISTER_2_VERTICAL_INTERRUPT_ENABLE       = %00100000
VDP_REGISTER_2_DMA_ENABLE                      = %00010000
VDP_REGISTER_2_PAL_ONLY_240_RESOLUTION_ENABLE  = %00001000
VDP_REGISTER_2_MEGA_DRIVE_VIDEO_MODE_ENABLE    = %00000100 ; SEGA says always one.

VDP_REGISTER_3_TH_LIGHTGUN_INTERRUPT_ENABLE    = %00001000
VDP_REGISTER_3_VERTICAL_SCROLL_2_CELLS         = %00000100
VDP_REGISTER_3_VERTICAL_SCROLL_COMPLETELY      = %00000000
VDP_REGISTER_3_HORIZONTAL_SCROLL_COMPLETELY    = %00000000
VDP_REGISTER_3_HORIZONTAL_SCROLL_8_PIXELS      = %00000010
VDP_REGISTER_3_HORIZONTAL_SCROLL_1_PIXELS      = %00000011

VDP_REGISTER_4_320_PIXEL_WIDE_DISPLAY_ENABLE   = %10000001
VDP_REGISTER_4_VSYNC_IS_NOW_PIXEL_CLOCK_ENABLE = %01000000 ; SEGA says always zero.
VDP_REGISTER_4_HORIZONTAL_SYNC_FREEZE_ENABLE   = %00100000 ; SEGA says always zero.
VDP_REGISTER_4_EXTERNAL_PIXEL_BUS_ENABLE       = %00010000 ; SEGA says always zero.
VDP_REGISTER_4_SHADOW_HIGHLIGHT_MODE_ENABLE    = %00001000
VDP_REGISTER_4_INTERLACE_ENABLE                = %00000010
VDP_REGISTER_4_SONIC_2_DOUBLE_INTERLACE_ENABLE = %00000110

                           ;FEDCBA9876543210FEDBCA9876543210
                           ;10AAAAAAaaaaaaaaXXXXXXXX5432xxAA
VDP_LCOMMAND_VRAM_READ   = %00000000000000000000000000000000
VDP_LCOMMAND_VRAM_WRITE  = %01000000000000000000000000000000

VDP_LCOMMAND_VSRAM_READ  = %00000000000000000000000000010000
VDP_LCOMMAND_VSRAM_WRITE = %01000000000000000000000000010000

VDP_LCOMMAND_CRAM_READ   = %00000000000000000000000000100000
VDP_LCOMMAND_CRAM_WRITE  = %11000000000000000000000000000000

VDP_PRIORITY  = %1000000000000000
VDP_PALETTE_0 = %0000000000000000
VDP_PALETTE_1 = %0010000000000000
VDP_PALETTE_2 = %0100000000000000
VDP_PALETTE_3 = %0110000000000000
VDP_VFLIP     = %0001000000000000
VDP_HFLIP     = %0000100000000000

VDP_SPRITE_VSIZE__8 = %0000000000000000
VDP_SPRITE_VSIZE_16 = %0000000100000000
VDP_SPRITE_VSIZE_24 = %0000001000000000
VDP_SPRITE_VSIZE_32 = %0000001100000000
VDP_SPRITE_HSIZE__8 = %0000000000000000
VDP_SPRITE_HSIZE_16 = %0000010000000000
VDP_SPRITE_HSIZE_24 = %0000100000000000
VDP_SPRITE_HSIZE_32 = %0000110000000000
        
VDP_REGISTER_WINDOW_HUGS_L_EDGE = %00000000
VDP_REGISTER_WINDOW_HUGS_R_EDGE = %10000000
VDP_REGISTER_WINDOW_HUGS_T_EDGE = %00000000
VDP_REGISTER_WINDOW_HUGS_B_EDGE = %10000000
        
PALETTE_SIZE_WORD_ENTRIES = 16
PALETTE_SIZE_BYTES        = (2*PALETTE_SIZE_WORD_ENTRIES)

; Top left of sprite-space to make calculations more obvious.
; The huge extra area is to allow sprites to slide off-screen.
; It's 128 in both dimensions.
SPRITE_TL = 128

; VRAM write address is swizzled up within the write longword:
;
; 31 30 29 28 27 26 25 24 23 22 21 20 19 18 17 16 15 14 13 12 11 10  9  8  7  6  5  4  3  2  1  0
;  7  6  5  4  3  2  1  0  7  6  5  4  3  2  1  0  7  6  5  4  3  2  1  0  7  6  5  4  3  2  1  0         
;      A13A12A11A10 A9 A8 A7 A6 A5 A4 A3 A2 A1 A0                                          A15A14

;VDP_LCOMMAND_CRAM_WRITE_PALETTE_LINE_X: dont think these will be useful after all
;        dc.l    (VDP_LCOMMAND_CRAM_WRITE|(0<<21))
;        dc.l    (VDP_LCOMMAND_CRAM_WRITE|(1<<21))
;        dc.l    (VDP_LCOMMAND_CRAM_WRITE|(2<<21))
;        dc.l    (VDP_LCOMMAND_CRAM_WRITE|(3<<21))
        
; vram_write:
; write_directive is a constant like VDP_LCOMMAND_VRAM_WRITE
; vram_address is a value from $0000-$FFFF.
; Produces a move.l writing the given constant address to MM_VDP_CONTROL, performs the
; weird swizzling for you.
        macro vram_write write_directive,vram_address        
        move.l  #((write_directive)|((((vram_address) & %0011111111111111) << 16) | (((vram_address) & %1100000000000000) >> 14))),(MM_VDP_CONTROL)
        endm        
        
; As above, but put the result in register.        
        macro vram_write_dX write_directive,vram_address,dX
        move.l  #((write_directive)|((((vram_address) & %0011111111111111) << 16) | (((vram_address) & %1100000000000000) >> 14))),dX
        endm        
        
; Disable interrupts: set supervisor bit and set mask to 7 to disable all interrupts.
; You MUST disable interrupts before attempting a !TIME write since the vector table will
; cease to exist when the cart bus is isolated.
        macro interrupts_disable
        move    #%0010011100000000,SR                  
        endm
        
; Enable  interrupts: set supervisor bit and set mask to 3 to disable unused interrupts. (Allowing v and hblank.)
        macro interrupts_enable
        move    #%0010001100000000,SR                  
        endm

        macro z80_bus_request_engage
        move.w  #$0100,(MM_Z80_BUS_REQUEST)
        endm

        macro z80_wait_bus_available
l\?     btst    #0,(MM_Z80_BUS_REQUEST)
        bne     l\?
        endm

        macro z80_bus_request_disengage
        move.w  #$0000,(MM_Z80_BUS_REQUEST)
        endm

        macro z80_reset_engage
        move.w  #$0000,(MM_Z80_RESET)
        endm

        macro z80_reset_disengage
        move.w  #$0100,(MM_Z80_RESET)
        endm

; wait_for_vcounter_equals:
; Spinlocks until vcounter = line.
; Invalidates d0.
        macro wait_for_vcounter_equals line
loop\?: move.w  (MM_VDP_HV_COUNTER),d0
        and.w   #$FF00,d0
        cmp.w   #(line<<8),d0
        bne     loop\?
        endm
        
        macro YM2612_wait
again\? btst    #YM2612_STATUS_BYTE_BUSY_BIT,(MM_YM2612_ADDRESS_P1)
        bne     again\?
        endm

; Write the 16-bit value in d0.w to TIME, activating the Game Raccoon cart.
; This isolates the cart from the bus, so any ROM operations will fail at this point!
        macro TIME_write_d0_w
        move.w  d0,(MM_TIME_ADDRESS_BEGIN)
        ; This delay needs to be at least as long as the maximum response time
        ; of the Game Raccoon cart: the interrupt being serviced and jumped to,
        ; the PORT managing instruction being reached, executed and the physical
        ; state of the pin changing. Need to read up the possible size of this
        ; delay and compare it to the 68000 clock speed and then give a HUGE
        ; margin.
        
        ; PIC: 32Mhz FOSC, 16Mhz FCY
        ; 4 FCY to service the interrupt, 11 FCY within the interrupt
        ; then another 20 FCY to navigate to the isolation function
        ; and call it. 35 FCY / 16Mhz FCY = 2.1875 us
        ; PAL Mega Drive: 7.600489 Mhz
        ; nop = 4(1/0) cycles execution
        ; 7.600489Mhz * 4 = 0.52628 us per NOP
        ; 2.1875 us / 0.52628 us = 4.16 NOPs required.
        nop ; 1
        nop ; 2
        nop ; 3
        nop ; 4
        nop ; 5 
        nop ; 6
        nop ; 7
        nop ; 8
        nop ; 9
        nop ; 10
        endm

; Read the 16-bit value from the register mapped to TIME in d0.w.
; Hope this works!
        macro TIME_read_d0_w
        move.w  (MM_TIME_ADDRESS_BEGIN),d0
        endm

; Perform a test to see if isolation is currently active.
; Result:
; ZERO FLAG/EQUAL CONDITION: Non-isolation.
;        NONEQUAL CONDITION: Isolated.
        macro GAME_RACCOON_isolation_test
        move.l  (CART_STACK_POSITION),d0
        and.l   #1,d0                        ; The cart is not isolated if this can be zero.
        endm
; TODO: Can I do btst #0,CART_STACK_POSITION+3 to save registers and space?        