; Game Raccoon raccoonramresident.68k.asm
; Game Raccoon Revision 1 68000 Menu Firmware
; (c) Mathew Carr July 2020
;
; The purpose of this file is to automate various inclusion tasks
; to allow imported files to have associated location and size
; symbols, and trigger the generation of static screens such as
; the predefined tegel information screens.
;
; Copyright (c) 2020 Mathew Carr
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU Affero General Public License Version 3
; with Additional Terms as found in the file
; LICENSE_AGPL_V3_ADDITIONAL_TERMS.txt.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU Affero General Public License for more details.
; 
; You should have received a copy of the GNU Affero General Public License
; along with this program.  If not, see <https://www.gnu.org/licenses/>.
;
; Use of the Game Raccoon logotype, name, characters and other identifying
; material is subject to license conditions.
;
; Revision 0 started:
; 25th December 2019.
;
; Revision 1 begins with aplomb:
; 27th April 2020.
;
; Good menu more or less complete:
; 10th May 2020.
; Needs raccoon animations, some cleaning up,
; vcounter work (it's waiting on a wrong line,
; and allowing the logic to run at 100Hz
; occasionally), but it's stable and logically
; fine, communicating with the PIC a treat.
;
; 15th May 2020.
; Fixed the wrong line waiting and sound bug!
;
; This file contains the RAM-resident part of the Game Raccoon
; menu firmware. Part of the Game Raccoon menu firmware must be
; RAM-resident to allow it to be executed while the cartridge
; is isolated and the BOOT parition is unavailable.
;
; 20th June 2020.
; With the updated boot sequence that clears the IO state, lots of
; Sonic mods come to life, hooray!
;

; PAL.
FRAMES_PER_SECOND = 50

; Magic word values used to communicate with the PIC:
GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_GR_HANDSHAKE                = $4752 ; Handshake to enable Game Raccoon PIC on boot. == "GR" in ASCII.
GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_REFLASH_MENU_FIRMWARE       = $FF00 ; Intstall new firmware from predefined string location.
GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_REGENERATE_DIRECTORY_CACHE  = $FFFE ; Regenerate the directory cache, returns 0 on success, else on failure.
GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_BOOT_CURRENT_GAME_PARTITION = $FFFF ; Boot the current game. No return value obviously.

; Copy these from PIC firmware main.c
REGISTERCLK_RESPONSE_GR_HANDSHAKE_OK                                = 0x0000
REGISTERCLK_RESPONSE_GR_HANDSHAKE_SD_CARD_ERROR                     = 0x0001
REGISTERCLK_RESPONSE_GR_HANDSHAKE_NO_CONTENTS                       = 0x0002
REGISTERCLK_RESPONSE_GR_HANDSHAKE_UNKNOWN                           = 0x0003

REGISTERCLK_RESPONSE_REGENERATE_DIRECTORY_CACHE_OK                  = 0x0000
REGISTERCLK_RESPONSE_REGENERATE_DIRECTORY_CACHE_ERROR               = 0x0001
REGISTERCLK_RESPONSE_REGENERATE_DIRECTORY_CACHE_STRING_TOO_LONG     = 0x0002
REGISTERCLK_RESPONSE_REGENERATE_DIRECTORY_CACHE_LIMIT_REACHED_ERROR = 0x0003

REGISTERCLK_RESPONSE_INSTALL_GAME_OK                                = 0x0000
REGISTERCLK_RESPONSE_INSTALL_GAME_FILE_TOO_BIG                      = 0x0001
REGISTERCLK_RESPONSE_INSTALL_GAME_READ_ERROR                        = 0x0002
REGISTERCLK_RESPONSE_INSTALL_GAME_VERIFY_ERROR                      = 0x0003

; You MUST change these enables to reflect whether you're compiling for
; real hardware (=0) or an emulator (=1):
; At present, there are no emulators that simulate the Game Raccoon
; cartridge's hardware, so any reads from TIME addresses will be
; emulator-dependent. Use enables to force a specific response.
;
; As emulators can't emulate isolation, we can simulate it in code
; with conditional assembly. Specify how long the isolation should
; last for for each isolation state.
; If this is not enabled, the isolation test passes instantly
; since the valid value remains accessible constantly.
;
; If you enable these and use the resulting binary on hardware,
; you'll be ignoring the read error/success codes from the register.
; You WON'T pop out of isolation prematurely, that test is still in
; place.
SIMULATE_TIME_REGISTER_READ_HANDSHAKE_ENABLE                       = 1
SIMULATE_TIME_REGISTER_READ_HANDSHAKE_RESPONSE                     = REGISTERCLK_RESPONSE_GR_HANDSHAKE_OK
SIMULATE_ISOLATION_DELAY_HANDSHAKE_ENABLE                          = 1
SIMULATE_ISOLATION_DELAY_HANDSHAKE_LENGTH_FRAMES                   = 1*FRAMES_PER_SECOND

SIMULATE_TIME_REGISTER_READ_REGENERATE_DIRECTORY_CACHE_ENABLE      = 1
SIMULATE_TIME_REGISTER_READ_REGENERATE_DIRECTORY_CACHE_RESPONSE    = REGISTERCLK_RESPONSE_REGENERATE_DIRECTORY_CACHE_OK
SIMULATE_ISOLATION_DELAY_REGENERATE_DIRECTORY_CACHE_ENABLE         = 1
SIMULATE_ISOLATION_DELAY_REGENERATE_DIRECTORY_CACHE_LENGTH_FRAMES  = 2*FRAMES_PER_SECOND

SIMULATE_TIME_REGISTER_READ_INSTALL_GAME_ENABLE                    = 1
SIMULATE_TIME_REGISTER_READ_INSTALL_GAME_RESPONSE                  = REGISTERCLK_RESPONSE_INSTALL_GAME_OK
SIMULATE_ISOLATION_DELAY_INSTALL_GAME_ENABLE                       = 1
SIMULATE_ISOLATION_DELAY_INSTALL_GAME_LENGTH_FRAMES                = 4*FRAMES_PER_SECOND

        ; The RAM resident code is compiled to run from $FF0000.
        ; It'll be copied into position by the entry point.
        ; At the address of $FF0000+CODELENGTH, the static RAM memory
        ; map with my global variables in begins.
        rorg    RAM_START
game_raccoon_ram_resident_assembled_block_begin:    

; game_raccoon_start_emergency:
; Do the handshake without affecting VDP, then boot game immediately.
game_raccoon_start_emergency:
        move.w  #GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_GR_HANDSHAKE,d0
        TIME_write_d0_w
emergency_handshake_loop_until_deisolated:
        GAME_RACCOON_isolation_test
        bne     emergency_handshake_loop_until_deisolated        
        jmp     game_raccoon_game_partition_boot_sequence
        
; game_raccoon_game_partition_boot_sequence:
; Signal to the PIC that we want to boot the currently installed game,
; wait for deisolation, install the anti-bootstrap sequence at the very
; end of RAM, jump to anti-bootstrap, erase all Game Raccoon RAM-resident
; and VRAM-resident data, and jump to game entry point.
game_raccoon_game_partition_boot_sequence:
        move.w  #GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_BOOT_CURRENT_GAME_PARTITION,d0
        TIME_write_d0_w
game_raccoon_game_partition_boot_sequence_loop_until_deisolated:
        GAME_RACCOON_isolation_test
        bne     game_raccoon_game_partition_boot_sequence_loop_until_deisolated
        ; Alright, we've switched to the game partition!
        ; We CANNOT return to the boot partition by any means now.
        ; All ROM-resident data is inaccessible.
        ; Clear VDP state and VRAM.
        jsr     vdp_clear_all_cram        
        jsr     vdp_clear_all_vram        
        jsr     vdp_clear_all_vsram      
        ; Additional boot state suggested by Sik and raq: http://gendev.spritesmind.net/forum/viewtopic.php?f=2&t=3163
        ; Hold the Z80 in reset.
        z80_bus_request_engage
        z80_reset_engage
        ; Clear the IO control registers (controller port directionality state).
        move.w  #0,d0
        move.w  d0,MM_CONTROLLER_1_CONTROL
        move.w  d0,MM_CONTROLLER_2_CONTROL
        move.w  d0,MM_EXPANSION_PORT_CONTROL 
        ; Write a zero to TIME which may help compatibility with some homebrew.
        ; This will be ignored by the PIC.
        TIME_write_d0_w
        ; Copy the anti-bootstrap to the end of RAM where the stack lives,
        ; destroying the stack as we go.
        ; The anti-bootstrap erases as much as it can and then finally
        ; jumps to the new game ROM image.        
        lea     game_raccoon_anti_bootstrap_start,a0
        lea     GAME_RACCOON_ANTI_BOOTSTRAP_DESTINATION,a1
        move.w  #(GAME_RACCOON_ANTI_BOOTSTRAP_BLOCK_COPY_LENGTH_LONGS-1),d0
game_raccoon_anti_bootstrap_copy_next_long:
        move.l  (a0)+,(a1)+
        dbf     d0,game_raccoon_anti_bootstrap_copy_next_long        
        jmp     GAME_RACCOON_ANTI_BOOTSTRAP_DESTINATION
        
; Anti-bootstrap:
; ---------------
; RAM Layout:
; START                                                                  END
; [RAM-resident Game Raccoon stuff] [RAM variables]                  [Stack]
; After anti-bootstrap installation:
; [RAM-resident Game Raccoon stuff] [RAM variables]         [Anti-bootstrap]
; After anti-bootstrap execution:
;                                                           [Anti-bootstrap]
; The new game should immediately clobber the anti-bootstrap's lingering stuff
; when it starts.        
        align   4
game_raccoon_anti_bootstrap_start:         
        ; Erase the Game Raccoon RAM-resident part up until the 
        ; game_raccoon_anti_bootstrap_erase_next_long label.
        lea     RAM_START,a0
        move.w  #(GAME_RACCOON_ANTI_BOOTSTRAP_ERASE_LENGTH_LONGS-1),d0
        move.l  #0,d1        
        jmp     GAME_RACCOON_ANTI_BOOTSTRAP_DESTINATION+game_raccoon_anti_bootstrap_erase_next_long-game_raccoon_anti_bootstrap_start
        align   4        
game_raccoon_anti_bootstrap_erase_next_long:        
        move.l  d1,(a0)+
        dbf     d0,game_raccoon_anti_bootstrap_erase_next_long
        ; Everything is erased except the previous two lines
        ; and the following three. HIT IT!
        move.l  (CART_STACK_POSITION),a7           ; Load the stack pointer starting value from the cart.
        move.l  (CART_ENTRY_POINT),a0              ; And jump to the entry point given in the cart image header!
        jmp     (a0)                            
game_raccoon_anti_bootstrap_end = *        
        
GAME_RACCOON_ANTI_BOOTSTRAP_BLOCK_COPY_LENGTH_LONGS = game_raccoon_anti_bootstrap_end-game_raccoon_anti_bootstrap_start
        
GAME_RACCOON_ANTI_BOOTSTRAP_DESTINATION = (RAM_END-(GAME_RACCOON_ANTI_BOOTSTRAP_BLOCK_COPY_LENGTH_LONGS*4)) 

GAME_RACCOON_ANTI_BOOTSTRAP_ERASE_LENGTH_LONGS = (((GAME_RACCOON_ANTI_BOOTSTRAP_DESTINATION+(game_raccoon_anti_bootstrap_erase_next_long-game_raccoon_anti_bootstrap_start))-RAM_START)/4)
; Anti-bootstrap stuff end.                
                             
; End of startup-related code.
; ----------------------------------------------------------
                             
; vdp_clear_all_vram:        
; Clear the entire contents of the ordinary VRAM. (As opposed to the CRAM or VSRAM.)
; Invalidates all registers.
vdp_clear_all_vram:
        ; Set up writing VRAM from VRAM start, incrementing by words.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$0000
        move.w  #(VRAM_SIZE_WORDS-1),d0
        move.w  #0,d1
vdp_clear_all_vram_next_word:
        move.w  d1,(MM_VDP_DATA)
        dbf     d0,vdp_clear_all_vram_next_word
        rts

; vdp_clear_all_vsram:
; Clear the entire contents of the VSRAM.
; Invalidates all registers.
vdp_clear_all_vsram:
        ; Set up writing VSRAM from VSRAM start, incrementing by words.
        vram_write VDP_LCOMMAND_VSRAM_WRITE,$0000
        move.w  #(VSRAM_SIZE_WORDS-1),d0
        move.w  #0,d1
vdp_clear_all_vsram_next_word:
        move.w  d1,(MM_VDP_DATA)
        dbf     d0,vdp_clear_all_vsram_next_word
        move.l  #0,(MM_VDP_CONTROL) ; Need to clear the second word of VDP_CONTROL for expected behaviour.
        rts

; vdp_clear_all_cram:
; Clear the CRAM to black for all four palettes.
; Invalidates all registers.
vdp_clear_all_cram:
        ; Set up writing CRAM from CRAM start, incrementing by words.
        vram_write VDP_LCOMMAND_CRAM_WRITE,$0000
        move.w  #(CRAM_SIZE_WORDS-1),d0
        move.w  #$0000,d1
vdp_clear_all_cram_next_word:
        move.w  d1,(MM_VDP_DATA)
        dbf     d0,vdp_clear_all_cram_next_word
        rts
        
; ----------------------------------------------------------

; intro_screen:
; This is the first screen that happens when the Game Raccoon is turned on.
; Fades from black to show the logo and some text and maybe a cute raccoon sprite.
; Performs the handshake and enters isolation until it is completed.
; When the handshake returns, reads the register to find the status of the PIC.
; May change colour and do other things based on error condition.
; Error condition will let you BOOT but recommend you restart. (use the power switch!)
; If there's no error condition, options are BOOT, BROWSE and ABOUT.

PALETTE_FADE_SPEED_SLOW = 4
PALETTE_FADE_SPEED_FAST = 16

intro_screen_begin:        
        jsr     plane_b_scrolling_background_initialise
        move.l  #$FFFFFFFF,plane_b_scrolling_background_status_struct_scroll_speed        
                
        move.l  #palette_black,palette_fade_0_ptr_a
        move.l  #palette_plane_b_lightblue_lightblue,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_SLOW,palette_fade_0_fade_speed
        
        move.l  #palette_black,palette_fade_1_ptr_a
        move.l  #palette_game_raccoon_logo_blue_font,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        
        move.l  #intro_screen_handle_state_fading_in,intro_screen_state
        
intro_screen_loop:
        wait_for_vcounter_equals 201
        wait_for_vcounter_equals 224
        lea     pad_state,a0
        jsr     controllers_read_and_derive
        
        jsr     plane_b_scrolling_background_update
        jsr     palette_fade_struct_update
        
        move.l  intro_screen_state,a0
        jsr     (a0)

        jmp     intro_screen_loop

intro_screen_handle_state_fading_in:
        move.l  palette_fade_0_fade_speed,d0
        bne     intro_screen_handle_state_fading_in_not_finished
        lea     string_intro_handshaking,a0
        jsr     intro_screen_write_status_string
        move.w  #GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_GR_HANDSHAKE,d0
        TIME_write_d0_w
        IF SIMULATE_ISOLATION_DELAY_HANDSHAKE_ENABLE = 1
        move.l  #SIMULATE_ISOLATION_DELAY_HANDSHAKE_LENGTH_FRAMES,simulated_isolation_timer
        ENDIF
        move.l  #intro_screen_handle_state_handshake_in_progress,intro_screen_state
intro_screen_handle_state_fading_in_not_finished:
        rts
                
; Bitmask for choices available on choice screen.        
CHOICE_AVAILABLE_BOOT_BIT   = 0
CHOICE_AVAILABLE_BROWSE_BIT = 1
CHOICE_AVAILABLE_ABOUT_BIT  = 2

CHOICE_AVAILABLE_BOOT   = (1<<CHOICE_AVAILABLE_BOOT_BIT)
CHOICE_AVAILABLE_BROWSE = (1<<CHOICE_AVAILABLE_BROWSE_BIT)
CHOICE_AVAILABLE_ABOUT  = (1<<CHOICE_AVAILABLE_ABOUT_BIT)
        
intro_screen_handle_state_handshake_in_progress:
        ; Only check for isolation.        
        IF SIMULATE_ISOLATION_DELAY_HANDSHAKE_ENABLE = 1
        subi.l  #1,simulated_isolation_timer                                          ; Isolation ends when this timer reaches zero.
        ELSE
        GAME_RACCOON_isolation_test
        ENDIF
        beq     intro_screen_handle_state_handshake_in_progress_isolation_over
intro_screen_handle_state_handshake_in_progress_isolation_active:        
        ; Can't do anything while isolation is still active.
        rts
intro_screen_handle_state_handshake_in_progress_isolation_over:
        ; Check result and do stuff based on it.        
        move.l  #0,d0                                                                 ; Clear upper word of d0
        IF SIMULATE_TIME_REGISTER_READ_HANDSHAKE_ENABLE = 1
        move.w  #SIMULATE_TIME_REGISTER_READ_HANDSHAKE_RESPONSE,d0                    ; d0.w contains a -simulated- Game Raccoon Handshake result.
        ELSE
        TIME_read_d0_w                                                                ; d0.w contains a Game Raccoon Handshake result.
        ENDIF
        ; Write the string based on the handshake return value.
        move.l  d0,d1
        lsl.l   #2,d1
        lea     intro_screen_handshake_response_string_table,a0
        lea     (a0,d1),a0
        move.l  (a0),a0
        jsr     intro_screen_write_status_string
        ; Zero is OK, everything else is an error.
        move.l  d0,d0
        beq     intro_screen_handle_state_handshake_in_progress_result_ok
intro_screen_handle_state_handshake_in_progress_result_failure:        
        ; Show error, START symbol and BOOT option.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000
        move.w  #SPRITE_TL+176,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|0),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+48),(MM_VDP_DATA)
        move.w  #SPRITE_TL+160-34,(MM_VDP_DATA)
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #21,d1                                                                ; d1.l: X
        move.l  #23,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_boot,a0                                                        ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        move.l  #intro_screen_handle_state_ready_for_choice,intro_screen_state
        move.l  #CHOICE_AVAILABLE_BOOT,intro_screen_state_choices_available
        ; Change the background to be fancy.
        move.l  #palette_plane_b_red_red,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        rts        
        
intro_screen_handle_state_handshake_in_progress_result_ok:
        ; Show status, START BOOT, B BROWSE, C ABOUT.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000
        move.w  #SPRITE_TL+176,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|1),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+48),(MM_VDP_DATA)
        move.w  #SPRITE_TL+14,(MM_VDP_DATA)
        move.w  #SPRITE_TL+176,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|2),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+16),(MM_VDP_DATA)
        move.w  #SPRITE_TL+114,(MM_VDP_DATA)
        move.w  #SPRITE_TL+176,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|0),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+32),(MM_VDP_DATA)
        move.w  #SPRITE_TL+227,(MM_VDP_DATA)
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #7,d1                                                                 ; d1.l: X
        move.l  #23,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_boot,a0                                                        ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        move.l  #19,d1                                                                ; d1.l: X
        lea     string_browse,a0                                                      ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        move.l  #33,d1                                                                ; d1.l: X
        lea     string_about,a0                                                       ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        move.l  #intro_screen_handle_state_ready_for_choice,intro_screen_state
        move.l  #(CHOICE_AVAILABLE_BOOT|CHOICE_AVAILABLE_BROWSE|CHOICE_AVAILABLE_ABOUT),intro_screen_state_choices_available
        rts
        
intro_screen_handle_state_ready_for_choice:
        btst    #CHOICE_AVAILABLE_BOOT_BIT,intro_screen_state_choices_available+3
        beq     intro_screen_handle_state_ready_for_choice_not_boot        
intro_screen_handle_state_ready_for_choice_try_boot:
        btst    #CONTROLLER_S_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     intro_screen_handle_state_ready_for_choice_not_boot
        ; Transition to boot.
        move.l  #palette_black,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_black,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_black,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        move.l  #intro_screen_handle_state_fading_to_boot,intro_screen_state
        rts        
intro_screen_handle_state_ready_for_choice_not_boot:
        btst    #CHOICE_AVAILABLE_BROWSE_BIT,intro_screen_state_choices_available+3
        beq     intro_screen_handle_state_ready_for_choice_not_browse
intro_screen_handle_state_ready_for_choice_try_browse:
        btst    #CONTROLLER_B_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     intro_screen_handle_state_ready_for_choice_not_browse
        ; Transition to browse.
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        move.l  #intro_screen_handle_state_fading_to_browse,intro_screen_state
        rts        
intro_screen_handle_state_ready_for_choice_not_browse:
        btst    #CHOICE_AVAILABLE_ABOUT_BIT,intro_screen_state_choices_available+3
        beq     intro_screen_handle_state_ready_for_choice_not_about
intro_screen_handle_state_ready_for_choice_try_about:
        btst    #CONTROLLER_C_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     intro_screen_handle_state_ready_for_choice_not_about
        ; Transition to about.
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        move.l  #intro_screen_handle_state_fading_to_about,intro_screen_state
        rts
intro_screen_handle_state_ready_for_choice_not_about:
        rts
        
intro_screen_handle_state_fading_to_boot:
        move.l  palette_fade_0_fade_speed,d0
        bne     intro_screen_handle_state_fading_to_boot_not_finished
        jmp     game_raccoon_game_partition_boot_sequence
intro_screen_handle_state_fading_to_boot_not_finished:
        rts
        
intro_screen_handle_state_fading_to_about:
        move.l  palette_fade_0_fade_speed,d0
        bne     intro_screen_handle_state_fading_to_about_not_finished
        jmp     about_screen_begin
intro_screen_handle_state_fading_to_about_not_finished:
        rts
        
intro_screen_handle_state_fading_to_browse:
        move.l  palette_fade_0_fade_speed,d0
        bne     intro_screen_handle_state_fading_to_browse_not_finished
        ;jmp browse_screen_begin; Which screen should we jump to? Should we skip the regeneration?
        jmp     regeneration_screen_begin
intro_screen_handle_state_fading_to_browse_not_finished:
        rts
        
; intro_screen_write_status_string:
; Takes a0 as string to write.
; No registers invalidated.     
intro_screen_write_status_string:
        movem.l d0-d4/a0-a1,-(a7)

        ; Need to clear the rows containing the text first.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000+18*64*2
        move.w  #(128-1),d0
intro_screen_write_status_string_clear_next_tile:        
        move.w  #0,(MM_VDP_DATA)
        dbf     d0,intro_screen_write_status_string_clear_next_tile
        
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #18,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     draw_string_8x16,a1
        jsr     draw_string_centred_a1
        
        movem.l (a7)+,d0-d4/a0-a1
        rts

; - END OF INTRO SCREEN -
; -----------------------

; about_screen:
; Credits and things. Also a sneaky way to make sure that the
; 'flash internal menu firmware' option is well hidden.

; Display this number of lines on the about screen.
; Does not include the blank row at the top.
; We're using the Window plane again for madness.
; ('again' as in this was added after the browse scrolling)
ABOUT_SCREEN_DISPLAYED_LINES_COUNT = 21

about_screen_begin:        
        jsr     plane_b_scrolling_background_initialise
        move.l  #$FFFFFFFF,plane_b_scrolling_background_status_struct_scroll_speed        
                
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_a
        move.l  #palette_plane_b_darkblue_darkblue,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_SLOW,palette_fade_0_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_a
        move.l  #palette_game_raccoon_logo_blue_font,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_a
        move.l  #palette_game_raccoon_logo_yellow_font,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_SLOW,palette_fade_2_fade_speed
        
        ; Write the about screen text from a tegel blob in ROM.
        ; First, clear Plane A entirely.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000
        move.w  #0,d0        
        move.w  #(64*32)-1,d1
about_screen_begin_clear_next_tegel:
        move.w  d0,(MM_VDP_DATA)        
        dbf     d1,about_screen_begin_clear_next_tegel        
        ; Write the first ABOUT_SCREEN_DISPLAYED_LINES_COUNT rows.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000                                      ; First row of Plane A, use scrolling to offset it down on screen.
        lea     about_screen_predefined_tegels_bin+4,a0                               ; a0 points to predefined tegels data.
        move.w  #((64*ABOUT_SCREEN_DISPLAYED_LINES_COUNT)-1),d0
about_screen_begin_set_next_tegel:
        move.w  (a0)+,(MM_VDP_DATA)        
        dbf     d0,about_screen_begin_set_next_tegel
        
        ; Write BROWSE button: sprite into list, caption into Window plane.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000                                      ; Sprite table in VRAM
        move.w  #SPRITE_TL+176+8,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|1),(MM_VDP_DATA)            ; Link to sprite 1, the up widget.
        move.w  #(VDP_PALETTE_1|1189+16),(MM_VDP_DATA)
        move.w  #SPRITE_TL+114,(MM_VDP_DATA)
        move.l  #1,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #19,d1                                                                ; d1.l: X
        move.l  #23+1,d2                                                              ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_browse,a0                                                      ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        
        ; Reset START holding firmware update counter.
        move.l  #about_screen_handle_state_fading_in,about_screen_state
        move.l  #0,about_screen_start_hold_counter
        
        ; Reset about screen scroll value.
        move.l  #0,about_screen_scroll_current_position_pixels
        ; Calculate maximum scroll value (pixel deflection) from stored number of lines.
        ; = (no lines - no displayable) * 8
        move.l  #0,d0
        move.w  about_screen_predefined_tegels_bin,d0
        sub.w   #ABOUT_SCREEN_DISPLAYED_LINES_COUNT,d0
        asl.w   #3,d0
        move.l  d0,about_screen_scroll_maximum_position_pixels
        
about_screen_loop:
        ; Middle screen trigger enables Window display on beginning on row (1+DISPLAYED_LINES) of screen.
        wait_for_vcounter_equals 112                      
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|VDP_REGISTER_WINDOW_HUGS_B_EDGE+(1+ABOUT_SCREEN_DISPLAYED_LINES_COUNT)),(MM_VDP_CONTROL)
        
        ; Lower screen (during VBlank) trigger enables Window display on upper first row of screen.
        wait_for_vcounter_equals (224)
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|VDP_REGISTER_WINDOW_HUGS_T_EDGE+(1)),(MM_VDP_CONTROL)
        
        lea     pad_state,a0
        jsr     controllers_read_and_derive
        
        jsr     about_screen_interface_update_plane_a_scroll_register
        
        jsr     plane_b_scrolling_background_update
        jsr     palette_fade_struct_update
        
        jsr     about_screen_position_sprite_arrow_widgets
        
        move.l  about_screen_state,a0
        jsr     (a0)
        
        jmp     about_screen_loop

about_screen_handle_state_fading_in:
        move.l  palette_fade_0_fade_speed,d0
        bne     about_screen_handle_state_fading_in_not_finished
        move.l  #about_screen_handle_state_waiting,about_screen_state
about_screen_handle_state_fading_in_not_finished:
        rts

        ; Static interactive screen state.
about_screen_handle_state_waiting:
        ; Test for START:
        btst    #CONTROLLER_S_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     about_screen_handle_state_waiting_start_not_pressed
        ; If START is held, increment the counter.
        addi.l  #1,about_screen_start_hold_counter
        move.l  about_screen_start_hold_counter,d0
        and.l   #128,d0
        beq     about_screen_handle_state_waiting_return
        jmp     flash_warning_screen_begin
        rts
about_screen_handle_state_waiting_start_not_pressed:
        ; If START is idle, throw away accrued hold time.
        move.l  #0,about_screen_start_hold_counter
        
        ; Test for Up and Down.
        btst    #CONTROLLER_D_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     about_screen_handle_state_waiting_not_down
        ; If the scroll value is not maximum, allow scroll down.
        move.l  about_screen_scroll_maximum_position_pixels,d1
        move.l  about_screen_scroll_current_position_pixels,d0
        cmp.l   d0,d1
        beq     about_screen_handle_state_waiting_down_scroll_prohibited
        add.l   #1,d0
        move.l  d0,about_screen_scroll_current_position_pixels
        ; If the scroll value moves onto a new cell (s&7 == 1), draw the next row.
        move.b  d0,d2
        andi.b  #7,d2
        cmpi.b  #1,d2
        bne     about_screen_handle_state_waiting_down_scroll_not_drawing_new_row        
        ; Draw row ((current / 8) + displayable) at that row & 31 into Plane A.
        asr.l   #3,d0
        add.l   #ABOUT_SCREEN_DISPLAYED_LINES_COUNT,d0                              
        jsr     about_screen_write_scrolling_prefined_tegels_row_d0                        
about_screen_handle_state_waiting_down_scroll_not_drawing_new_row:
about_screen_handle_state_waiting_down_scroll_prohibited:        
about_screen_handle_state_waiting_not_down:
        btst    #CONTROLLER_U_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     about_screen_handle_state_waiting_not_up
        ; If the scroll value is not minimum (0), allow scroll up.
        move.l  about_screen_scroll_current_position_pixels,d0
        beq     about_screen_handle_state_waiting_up_scroll_prohibited
        sub.l   #1,d0
        move.l  d0,about_screen_scroll_current_position_pixels
        ; If the scroll value moves onto a new cell (s&7 == 7), draw the previous row.
        move.b  d0,d2
        andi.b  #7,d2
        cmpi.b  #7,d2
        bne     about_screen_handle_state_waiting_up_scroll_not_drawing_new_row        
        ; Draw row (current / 8) at that row & 31 into Plane A.
        asr.l   #3,d0                    
        jsr     about_screen_write_scrolling_prefined_tegels_row_d0                        
about_screen_handle_state_waiting_up_scroll_not_drawing_new_row:
about_screen_handle_state_waiting_up_scroll_prohibited:        
about_screen_handle_state_waiting_not_up:
        
        ; Test for Button B.
        btst    #CONTROLLER_B_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     about_screen_handle_state_waiting_not_browse
        ; Transition to browse.
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        move.l  #about_screen_handle_state_fading_to_browse,about_screen_state
        rts        
about_screen_handle_state_waiting_not_browse:
about_screen_handle_state_waiting_return:        
        rts

; about_screen_write_scrolling_prefined_tegels_row_d0:
; Write the row d0 of 64 tegels from about_screen_predefined_tegels_bin into Plane A.
; No registers invalidated.
about_screen_write_scrolling_prefined_tegels_row_d0:
        movem.l d0/d1/a0,-(a7)
        
        move.l  d0,d1
        
        andi.l  #31,d0                                                                ; Row of Plane A to write into.
        asl.l   #7,d0                                                                 ; Offset into Plane A to write into = row * (64*2).        
        add.l   #$C000,d0                                                             ; Construct full VRAM address in Plane A.
        jsr     vram_write_swizzle_address_in_d0
        move.l  d0,(MM_VDP_CONTROL)                                                   ; Write destination established.
                
        asl.l   #7,d1                                                                 ; Calculate d1 * 128 into d1.
        lea     about_screen_predefined_tegels_bin+4,a0
        lea     (a0,d1.l),a0                                                          ; a0 = tegel reading source from predefined tegels.
        
        ; Write 64 tegels from source.
        move.l  #(64-1),d0
about_screen_write_scrolling_prefined_tegels_row_d0_next_tegel:
        move.w  (a0)+,(MM_VDP_DATA)        
        dbf     d0,about_screen_write_scrolling_prefined_tegels_row_d0_next_tegel
        
        movem.l (a7)+,d0/d1/a0
        rts

; about_screen_interface_update_plane_a_scroll_register:
; Set the scroll value of Plane A to reflect the scroll position.
; No registers invalidated.
about_screen_interface_update_plane_a_scroll_register:
        movem.l d0,-(a7)
        
        vram_write (VDP_LCOMMAND_VSRAM_WRITE),$0000+0
        move.l  #(-(8*1)),d0
        add.l   about_screen_scroll_current_position_pixels,d0
        move.w  d0,(MM_VDP_DATA)
        
        movem.l (a7)+,d0
        rts
        
; about_screen_position_sprite_arrow_widgets:
; Place the arrow widgets if they are visible.
; Invalidates all registers.        
about_screen_position_sprite_arrow_widgets:        
        ; We're modifying sprite slots 1 (up) and -> 2 (down) which links to zero.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000+8*1
        
        ; Default X position of arrow widgets, common to both.
        move.w  #SPRITE_TL+287,d2
        ; Show 'up' conditionally, change the widgets visibility by modifying their X position (stored in d0).
        move.w  d2,d0
        ; Up is shown if the scroll current value is nonzero.
        move.l  about_screen_scroll_current_position_pixels,d1
        bne     about_screen_position_sprite_arrow_widgets_up_widget_visible
        move.w  #10,d0                                                                ; Set X to offscreen position (not special zero though)
about_screen_position_sprite_arrow_widgets_up_widget_visible:        
        move.w  #SPRITE_TL+16,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_16|VDP_SPRITE_VSIZE_16|2),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1181+0),(MM_VDP_DATA)
        move.w  d0,(MM_VDP_DATA)
                
        ; Show 'down' conditionally, change the widgets visibility by modifying their X position (stored in d0).
        move.w  d2,d0
        ; Down is shown if the scroll destination is != maximum possible.
        move.l  about_screen_scroll_maximum_position_pixels,d1
        move.l  about_screen_scroll_current_position_pixels,d2
        cmp.l   d2,d1
        bne     about_screen_position_sprite_arrow_widgets_down_widget_visible
        move.w  #10,d0                                                                ; Set X to offscreen position (not special zero though)
about_screen_position_sprite_arrow_widgets_down_widget_visible:        
        move.w  #SPRITE_TL+165,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_16|VDP_SPRITE_VSIZE_16|0),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1185+0),(MM_VDP_DATA)
        move.w  d0,(MM_VDP_DATA)        
        
        rts        
        
about_screen_handle_state_fading_to_browse:
        move.l  palette_fade_0_fade_speed,d0
        bne     about_screen_handle_state_fading_to_browse_not_finished
        jmp     regeneration_screen_begin
about_screen_handle_state_fading_to_browse_not_finished:
        rts        
        
; - END OF ABOUT SCREEN -
; -----------------------

; flash_warning_screen_begin:
; Show a warning before flashing the internal menu firmware from SD card.
flash_warning_screen_begin:        
        jsr     plane_b_scrolling_background_initialise
        move.l  #$FFFFFFFF,plane_b_scrolling_background_status_struct_scroll_speed        
                
        move.l  #palette_plane_b_darkblue_darkblue,palette_fade_0_ptr_a
        move.l  #palette_plane_b_red_red,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed        
        
        ; Write the flash warning screen text from a tegel blob in ROM.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000                                      ; Plane A.
        lea     flash_warning_screen_predefined_tegels_bin+2,a0                       ; +2 is the DBF loop counter from predefined tegels.
        move.w  (a0)+,d0                                                              ; d0 = DBF counter from predefined tegels, a0 = pointer to tegel data.
flash_warning_screen_begin_set_next_tegel:
        move.w  (a0)+,(MM_VDP_DATA)        
        dbf     d0,flash_warning_screen_begin_set_next_tegel
        ; Clear the sprite table too.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000                                      ; Sprite table in VRAM.
        move.w  #0,(MM_VDP_DATA)        
        move.w  #0,(MM_VDP_DATA)        
        
        ; Reset the vertical scrolling of Plane A.
        vram_write (VDP_LCOMMAND_VSRAM_WRITE),$0000+0
        move.w  #0,(MM_VDP_DATA)
        ; Disable window plane.
        move.w  #(VDP_REGISTER_WINDOW_PLANE_HORIZONTAL_EXTENT|$00),(MM_VDP_CONTROL)
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|$00),(MM_VDP_CONTROL)
        
        move.l  #flash_warning_screen_handle_state_fading_in,flash_warning_screen_state
        move.l  #0,flash_warning_screen_buttons_hold_counter
        
flash_warning_screen_loop:
        wait_for_vcounter_equals 201
        wait_for_vcounter_equals 224
        lea     pad_state,a0
        jsr     controllers_read_and_derive
        
        jsr     plane_b_scrolling_background_update
        jsr     palette_fade_struct_update
        
        move.l  flash_warning_screen_state,a0
        jsr     (a0)

        jmp     flash_warning_screen_loop

flash_warning_screen_handle_state_fading_in:
        move.l  palette_fade_0_fade_speed,d0
        bne     flash_warning_screen_handle_state_fading_in_not_finished
        move.l  #flash_warning_screen_handle_state_waiting,flash_warning_screen_state
flash_warning_screen_handle_state_fading_in_not_finished:
        rts

flash_warning_screen_handle_state_waiting:
        move.w   pad_state+PAD_1+CURR,d0
        and.w    #(CONTROLLER_A|CONTROLLER_B|CONTROLLER_C),d0
        bne      flash_warning_screen_handle_state_waiting_buttons_not_pressed
        ; If all buttons are held, increment the counter.
        addi.l  #1,flash_warning_screen_buttons_hold_counter
        move.l  flash_warning_screen_buttons_hold_counter,d0
        and.l   #128,d0
        beq     flash_warning_screen_handle_state_waiting_return
        ; We're going to transition to the game installation screen,
        ; except we're selecting the special firmware installation word!
        move.w  #GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_REFLASH_MENU_FIRMWARE,installation_screen_chosen_word
        move.l  #0,installation_screen_should_show_boot_on_success
        move.l  #((4*1024*1024)/(65536)),installation_screen_file_size_in_sectors     ; This is the size of the entire partition in terms of Flash format sectors. Should be 64.
        move.l  #flash_warning_screen_handle_state_fading_to_installation,flash_warning_screen_state
        
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        rts
flash_warning_screen_handle_state_waiting_buttons_not_pressed:
        ; If any button is idle, throw away accrued hold time.
        move.l  #0,flash_warning_screen_buttons_hold_counter
flash_warning_screen_handle_state_waiting_return:        
        rts

flash_warning_screen_handle_state_fading_to_installation:
        move.l  palette_fade_0_fade_speed,d0
        bne     flash_warning_screen_handle_state_fading_to_installation_not_finished
        jmp     installation_screen_begin
flash_warning_screen_handle_state_fading_to_installation_not_finished:
        rts        
                
; - END OF FLASH WARNING SCREEN -
; -------------------------------

; regeneration_screen:
; Fade from blue, and show the cute raccoon doing his thing.
; There's no interactivity or measurements on this screen,
; it's just there to pass the time while the SD card is being
; checked.

regeneration_screen_begin:        
        jsr     plane_b_scrolling_background_initialise
        move.l  #$FFFFFFFF,plane_b_scrolling_background_status_struct_scroll_speed        
                
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_a
        move.l  #palette_plane_b_darkblue_darkblue,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_a
        move.l  #palette_game_raccoon_logo_blue_font,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_a
        move.l  #palette_game_raccoon_logo_yellow_font,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        
        ; Clear the sprite table too.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000                                      ; Sprite table in VRAM
        move.w  #0,(MM_VDP_DATA)        
        move.w  #0,(MM_VDP_DATA)        
        
        ; Clear Plane A and write captions.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000                                      ; Plane A
        move.w  #((64*32)-1),d0
regeneration_screen_begin_set_next_tegel:
        move.w  #0,(MM_VDP_DATA)        
        dbf     d0,regeneration_screen_begin_set_next_tegel
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #2,d2                                                                 ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_regeneration_in_progress,a0                                    ; a0  : pointer to null-terminated string
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        
        ; Reset the vertical scrolling of Plane A.
        vram_write (VDP_LCOMMAND_VSRAM_WRITE),$0000+0
        move.w  #0,(MM_VDP_DATA)
        ; Disable window plane.
        move.w  #(VDP_REGISTER_WINDOW_PLANE_HORIZONTAL_EXTENT|$00),(MM_VDP_CONTROL)
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|$00),(MM_VDP_CONTROL)
                
        move.l  #regeneration_screen_handle_fade_state_fading_in,regeneration_screen_fade_state
        
        move.l  #0,regeneration_screen_raccoon_animation_state                        ; TODO: raccoon animation state?
        move.l  #0,regeneration_screen_raccoon_animation_frame                        ; TODO: raccoon animation state?
        
        ; Begin the actual regeneration here, before the animation starts
        ; while the screen is obscured.        
        ; Signal to the PIC that we want to regenerate the directory cache.
        move.l  #1,regeneration_screen_isolation_in_effect
        move.w  #GAME_RACCOON_TIME_WRITE_MAGIC_NUMBER_REGENERATE_DIRECTORY_CACHE,d0
        TIME_write_d0_w 
        IF SIMULATE_ISOLATION_DELAY_REGENERATE_DIRECTORY_CACHE_ENABLE = 1
        move.l  #SIMULATE_ISOLATION_DELAY_REGENERATE_DIRECTORY_CACHE_LENGTH_FRAMES,simulated_isolation_timer
        ENDIF
        ; We're in isolation now so we CANNOT refer to ROM routines
        ; or included ROM binaries until isolation is complete.
        ; TODO: All the raccoon frames need to be in VRAM or system RAM at this point.
                
regeneration_screen_loop:
        wait_for_vcounter_equals 201
        wait_for_vcounter_equals 224
        lea     pad_state,a0
        jsr     controllers_read_and_derive
        
        jsr     plane_b_scrolling_background_update
        jsr     palette_fade_struct_update
        
        jsr     regeneration_screen_animate_raccoon
        jsr     regeneration_screen_handle_isolation_scan
        
        move.l  regeneration_screen_fade_state,a0
        jsr     (a0)

        jmp     regeneration_screen_loop

regeneration_screen_handle_isolation_scan:
        ; Store isolation state in a flag and detect the transition from 1 to 0.
        cmpi.l  #0,regeneration_screen_isolation_in_effect
        beq     regeneration_screen_handle_installing_no_test
        IF SIMULATE_ISOLATION_DELAY_REGENERATE_DIRECTORY_CACHE_ENABLE = 1
        subi.l  #1,simulated_isolation_timer                                          ; Isolation ends when this timer reaches zero.
        ELSE
        GAME_RACCOON_isolation_test
        ENDIF
        bne     regeneration_screen_loop_isolation_still_in_effect
        jsr     regeneration_screen_regeneration_termination_event
regeneration_screen_loop_isolation_still_in_effect:
regeneration_screen_handle_installing_no_test:        
        ; No interface other than the raccoon here.
        
        ; Screen automatically ends when the conditions are met.
        ; As long as it isn't a failure!        
        rts        
        
regeneration_screen_regeneration_termination_event:
        ; Clear the old main caption.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000+2*64*2                               ; Plane A
        move.w  #((64*2)-1),d0
regeneration_screen_regeneration_termination_event_clear_old_title_next_tegel:
        move.w  #0,(MM_VDP_DATA)        
        dbf     d0,regeneration_screen_regeneration_termination_event_clear_old_title_next_tegel
        ; Do something with the change in isolation state from 1 to 0.
        ; Determine whether the regeneration failed or succeeded.
        move.l  #0,regeneration_screen_isolation_in_effect       
        move.l  #0,d0                                                                 ; Clear upper word of d0
        IF SIMULATE_TIME_REGISTER_READ_REGENERATE_DIRECTORY_CACHE_ENABLE = 1
        move.w  #SIMULATE_TIME_REGISTER_READ_REGENERATE_DIRECTORY_CACHE_RESPONSE,d0                ; d0.w contains a -simulated- Game Raccoon Regeneration response result.
        ELSE
        TIME_read_d0_w                                                                ; d0.w contains a Game Raccoon Regeneration response result.
        ENDIF
        move.w  d0,regeneration_screen_response_word_from_pic
        beq     regeneration_screen_regeneration_success_event
regeneration_screen_regeneration_failure_event:
        ; Write the appropriate failure caption.
        move.l  #0,d1                                                                 
        add.w   regeneration_screen_response_word_from_pic,d1                         ; d1 = failure code as long
        lsl.l   #2,d1                                                                 ; d1 = failure code * 4
        lea     regeneration_failure_caption_table-4,a0                               ; a0 = caption table moved by one entry
        move.l  (a0,d1),a0                                                            ; a0 = address of string to write
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #20,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        move.l  #2,d2                                                                 ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        lea     string_regeneration_failed,a0                                         ; a0  : pointer to null-terminated string
        jsr     draw_string_centred_a1        
        ; Red background.        
        move.l  #palette_plane_b_red_red,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        ; TODO Instruct the raccoon to play a sad animation.
        rts
        
regeneration_screen_regeneration_success_event: 
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #2,d2                                                                 ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_regeneration_complete,a0                                       ; a0  : pointer to null-terminated string
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        ; TODO Instruct the raccoon to play a happy animation.
        ; Begin the transition from the regeneration screen to the menu screen.
        ; Transition to menu.
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        move.l  #regeneration_screen_handle_fade_state_fading_to_browse,regeneration_screen_fade_state
        rts        
        
regeneration_screen_handle_fade_state_fading_in:
        move.l  palette_fade_0_fade_speed,d0
        bne     regeneration_screen_handle_fade_state_fading_in_not_finished
        move.l  #regeneration_screen_handle_fade_state_static,regeneration_screen_fade_state
regeneration_screen_handle_fade_state_fading_in_not_finished:
        rts
        
regeneration_screen_handle_fade_state_static:     
        rts
        
regeneration_screen_handle_fade_state_fading_to_browse:
        move.l  palette_fade_0_fade_speed,d0
        bne     regeneration_screen_handle_fade_state_fading_to_browse_not_finished
        jmp     browse_screen_begin
regeneration_screen_handle_fade_state_fading_to_browse_not_finished:
        rts
        
; ?!?!?!?!?!
; TODO!!!
; Magic asynchronous process whose state can be fiddled with by
; the other state handler?
regeneration_screen_animate_raccoon:
        rts
        
        align   4
regeneration_failure_caption_table:
        ; This table contains no entry for success (value zero), so subtract 4 bytes when indexing.
        dc.l    string_regeneration_failed_error    
        dc.l    string_regeneration_failed_string_too_long    
        dc.l    string_regeneration_failed_limit_reached    
        
; - END OF REGENERATION SCREEN -
; -----------------------

; installation_screen:
; Fade from blue, and show the cute raccoon doing his thing.
; There's going to be a progress bar, and a timer that'll
; show more or less how much time is remaining.
; The following variables are described at the end of this
; file and provide an interface to control the display of
; this screen before it's jumped to.
; installation_screen_chosen_word
; installation_screen_should_show_boot_on_success
; installation_screen_file_size_in_sectors

; "TiTAN - Overdrive (Rev1.1-106-Final) (Hardware).bin" is 3946282 bytes big,
; rounded up to the next 65536, that is 3997696 bytes = 61 Flash sectors.
; It takes 338 seconds to write this image on a Game Raccoon Revision 0.
; 338 seconds / 61 sectors = 5.54 seconds per sector.
; (338 seconds * 50 PAL frames per second) / 61 sectors = 278 PAL frames per sector.
INSTALLATION_FRAMES_PER_SECTOR = 278
; We don't use this value to detect completion or as a delay.
; If the isolation ends early because the heuristic is inaccurate or
; there's an error, then we trust reality and ignore whatever
; the variables say.

installation_screen_begin:        
        jsr     plane_b_scrolling_background_initialise
        move.l  #$FFFFFFFF,plane_b_scrolling_background_status_struct_scroll_speed        
                
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_a
        move.l  #palette_plane_b_darkblue_darkblue,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_a
        move.l  #palette_game_raccoon_logo_blue_font,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_a
        move.l  #palette_game_raccoon_logo_yellow_font,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        
        ; Reset the vertical scrolling of Plane A.
        vram_write (VDP_LCOMMAND_VSRAM_WRITE),$0000+0
        move.w  #0,(MM_VDP_DATA)
        
        ; Clear the sprite table too.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000                                      ; Sprite table in VRAM
        move.w  #0,(MM_VDP_DATA)        
        move.w  #0,(MM_VDP_DATA)        
        
        ; Disable window plane.
        move.w  #(VDP_REGISTER_WINDOW_PLANE_HORIZONTAL_EXTENT|$00),(MM_VDP_CONTROL)
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|$00),(MM_VDP_CONTROL)
        
        ; Clear Plane A and write captions.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000                                      ; Plane A
        move.w  #((64*32)-1),d0
installation_screen_begin_set_next_tegel:
        move.w  #0,(MM_VDP_DATA)        
        dbf     d0,installation_screen_begin_set_next_tegel
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #2,d2                                                                 ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_installation_in_progress,a0                                    ; a0  : pointer to null-terminated string
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        
        move.l  #installation_screen_handle_fade_state_fading_in,installation_screen_fade_state
        move.l  installation_screen_file_size_in_sectors,d0
        mulu.w  #INSTALLATION_FRAMES_PER_SECTOR,d0
        move.l  d0,installation_screen_expected_frames
        move.l  #0,installation_screen_elapsed_frames
        
        move.l  #0,installation_screen_choices_available
        
        move.l  #0,installation_screen_raccoon_animation_state                        ; TODO: raccoon animation state?
        move.l  #0,installation_screen_raccoon_animation_frame                        ; TODO: raccoon animation state?
        
        ; Begin the actual installation here, before the animation starts
        ; while the screen is obscured.        
        ; Signal to the PIC that we want to install the game whose
        ; filename resides in the directory cache record stored in d0.
        ; (Or it may the be menu firmware flash special word.)
        move.l  #1,installation_screen_isolation_in_effect
        move.w  installation_screen_chosen_word,d0
        TIME_write_d0_w 
        IF SIMULATE_ISOLATION_DELAY_INSTALL_GAME_ENABLE = 1
        move.l  #SIMULATE_ISOLATION_DELAY_INSTALL_GAME_LENGTH_FRAMES,simulated_isolation_timer
        ENDIF
        ; We're in isolation now so we CANNOT refer to ROM routines
        ; or included ROM binaries until isolation is complete.
        ; TODO: All the raccoon frames need to be in VRAM or system RAM at this point.
        
installation_screen_loop:
        wait_for_vcounter_equals 201
        wait_for_vcounter_equals 224
        lea     pad_state,a0
        jsr     controllers_read_and_derive
        
        jsr     plane_b_scrolling_background_update
        jsr     palette_fade_struct_update
        
        jsr     installation_screen_animate_raccoon
        jsr     installation_screen_handle_installing
        
        move.l  installation_screen_fade_state,a0
        jsr     (a0)

        jmp     installation_screen_loop

installation_screen_handle_installing:        
        ; Advance the elapsed frames up to the expected frames,
        ; but not beyond it.
        move.l  installation_screen_elapsed_frames,d0
        cmp.l   installation_screen_expected_frames,d0
        beq     installation_screen_loop_do_not_advance_elapsed_frames
        addi.l  #1,installation_screen_elapsed_frames
installation_screen_loop_do_not_advance_elapsed_frames:

        ; Store isolation state in a flag and detect the transition from 1 to 0.
        cmpi.l  #0,installation_screen_isolation_in_effect
        beq     installation_screen_handle_installing_no_test
        IF SIMULATE_ISOLATION_DELAY_INSTALL_GAME_ENABLE = 1
        subi.l  #1,simulated_isolation_timer                                          ; Isolation ends when this timer reaches zero.
        ELSE
        GAME_RACCOON_isolation_test
        ENDIF
        bne     installation_screen_loop_isolation_still_in_effect
        jsr     installation_screen_installation_termination_event
installation_screen_loop_isolation_still_in_effect:
installation_screen_handle_installing_no_test:        
       
        ; Update the progress bar and timer only during installation.
        cmpi.l  #0,installation_screen_isolation_in_effect
        beq     installation_screen_handle_installing_interface_complete
installation_screen_handle_installing_interface_in_progress:
        ; There are sixteen cells across the screen.
        ; I'm going to draw the progress bar as a string.
        ; First, construct the string unlit.
        ; TODO this design is ugly. BUT IT WORKS. It needs replacing if and
        ; when we get a raccoon graphic.
        lea     -16(a7),a7                                                            ; Acquire sixteen bytes from the stack.
        move.l  a7,a1
        move.l  a1,a0
        move.l  #'----',d0
        move.l  d0,(a1)+
        move.l  d0,(a1)+
        move.l  d0,(a1)+
        move.l  d0,(a1)+
        move.b  #0,(a1)
        ; Calculate the progress as a value from 0-16.
        ; Progress = FLOOR((elapsed_frames * 17) / (expected_frames+1))
        ; This function starts at 0, goes up to 16 but never crosses into 17.
        ; The maximum number of frames we can handle is 20000, which
        ; is way more than 64 sectors ought to take, but still in MULU range.
        move.l  installation_screen_elapsed_frames,d0
        mulu.w  #17,d0
        move.l  installation_screen_expected_frames,d1
        addi.l  #1,d1
        divu.w  d1,d0                                                                 ; d0 contains number of cells to light, in lower word.        
        beq     installation_screen_handle_installing_interface_in_progress_progress_bar_no_cells
        subi.w  #1,d0                                                                 ; d0 is dbf compatible loop counter.  
        move.l  a0,a1
        move.b  #'O',d1
installation_screen_handle_installing_interface_in_progress_progress_bar_next_cell:
        move.b  d1,(a1)+
        dbf     d0,installation_screen_handle_installing_interface_in_progress_progress_bar_next_cell
installation_screen_handle_installing_interface_in_progress_progress_bar_no_cells:
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #20,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        lea     16(a7),a7                                                             ; Return sixteen bytes to the stack
        ; Drawing progress bar string done.
        ; I'm going to put a nice timer below the progress bar.
        ; M:SS. The max time is 6:00, the colon should blink.
INSTALLATION_SCREEN_TIME_STRING_OFFSET_MINUTES      = 0
INSTALLATION_SCREEN_TIME_STRING_OFFSET_COLON        = 1
INSTALLATION_SCREEN_TIME_STRING_OFFSET_SECONDS_TENS = 2
INSTALLATION_SCREEN_TIME_STRING_OFFSET_SECONDS_ONES = 3
INSTALLATION_SCREEN_TIME_STRING_OFFSET_NULL         = 4
        ; Use the half-second measurement to calculate the timer, and the blinking colon.
INSTALLATION_FRAMES_PER_HALF_SECOND = 25
        lea     -16(a7),a7                                                            ; Acquire sixteen bytes from the stack
        move.l  a7,a0
        move.b  #0,INSTALLATION_SCREEN_TIME_STRING_OFFSET_NULL(a0)                    ; Init string to '-:--z'
        move.b  #':',INSTALLATION_SCREEN_TIME_STRING_OFFSET_COLON(a0)
        move.l  installation_screen_expected_frames,d0
        sub.l   installation_screen_elapsed_frames,d0                                 ; d0 is number of remaining frames
        divu.w  #INSTALLATION_FRAMES_PER_HALF_SECOND,d0                               ; d0 contains number of half-seconds
        btst.l  #0,d0                                                                 ; Alternates if colon is visible
        beq     installation_screen_handle_installing_interface_in_progress_dont_erase_colon
        move.b  #' ',INSTALLATION_SCREEN_TIME_STRING_OFFSET_COLON(a0)
installation_screen_handle_installing_interface_in_progress_dont_erase_colon:        
        move.l  #0,d1
        move.w  d0,d1                                                                 ; Disregard remainder part of DIVU        
        lsr.l   #1,d1                                                                 ; d1 contains time remaining in seconds
        
        divu.w  #60,d1                                                                ; d1 contains seconds : minutes in word halves
        add.w   #'0',d1                                                               ; d1 lower word contains minutes as ascii
        move.b  d1,INSTALLATION_SCREEN_TIME_STRING_OFFSET_MINUTES(a0)                 ; String = 'M:--z'
        move.w  #0,d1                                                                 ; Clear quotient part of DIVU
        swap    d1                                                                    ; d1 contains time remaining in only seconds below a minute
        
        divu.w  #10,d1                                                                ; d1 contains seconds : tens-of-seconds in word halves
        add.w   #'0',d1                                                               ; d1 lower word contains tens-of-second as ascii
        move.b  d1,INSTALLATION_SCREEN_TIME_STRING_OFFSET_SECONDS_TENS(a0)            ; String = 'M:S-z'
        swap    d1                                                                    ; d1 lower word contains only seconds below tens
        add.b   #'0',d1                                                               ; d0 contains unit seconds as ascii
        move.b  d1,INSTALLATION_SCREEN_TIME_STRING_OFFSET_SECONDS_ONES(a0)            ; String = 'M:SSz'
        
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #23,d2                                                                ; d2.l: Y
        move.l  #2,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        
        lea     16(a7),a7                                                             ; Return sixteen bytes to the stack
        rts
        
installation_screen_handle_installing_interface_complete:                
        ; Detect installation stopped and respond to start?
        btst    #CHOICE_AVAILABLE_BOOT_BIT,installation_screen_choices_available+3
        beq     installation_screen_handle_installing_interface_complete_no_options
        btst    #CONTROLLER_S_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     installation_screen_handle_installing_interface_complete_not_boot
        ; Transition to boot.
        move.l  #palette_black,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_black,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #installation_screen_handle_fade_state_fading_to_boot,installation_screen_fade_state
        rts        
installation_screen_handle_installing_interface_complete_not_boot:
installation_screen_handle_installing_interface_complete_no_options:   
        rts
        
installation_screen_installation_termination_event:
        ; Clear the old main caption.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000+2*64*2                               ; Plane A
        move.w  #((64*2)-1),d0
installation_screen_installation_termination_event_clear_old_title_next_tegel:
        move.w  #0,(MM_VDP_DATA)        
        dbf     d0,installation_screen_installation_termination_event_clear_old_title_next_tegel
        ; Clear the timer region too.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000+2*64*23                              ; Plane A
        move.w  #((64*2)-1),d0
installation_screen_installation_termination_event_clear_timer_region_next_tegel:
        move.w  #0,(MM_VDP_DATA)        
        dbf     d0,installation_screen_installation_termination_event_clear_timer_region_next_tegel
        
        ; Do something with the change in isolation state from 1 to 0.
        ; Determine whether the installation failed or succeeded.
        move.l  #0,installation_screen_isolation_in_effect  
        move.l  #0,d0                                                                 ; Clear upper word of d0
        IF SIMULATE_TIME_REGISTER_READ_INSTALL_GAME_ENABLE = 1
        move.w  #SIMULATE_TIME_REGISTER_READ_INSTALL_GAME_RESPONSE,d0                              ; d0.w contains a -simulated- Game Raccoon Installation response result.
        ELSE
        TIME_read_d0_w                                                                ; d0.w contains a Game Raccoon Installation response result.
        ENDIF
        move.w  d0,installation_screen_response_word_from_pic
        beq     installation_screen_installation_success_event
installation_screen_installation_failure_event:
        ; Write the appropriate failure caption.
        move.l  #0,d1                                                                 
        add.w   installation_screen_response_word_from_pic,d1                         ; d1 = failure code as long
        lsl.l   #2,d1                                                                 ; d1 = failure code * 4
        lea     installation_failure_caption_table-4,a0                               ; a0 = caption table moved by one entry
        move.l  (a0,d1),a0                                                            ; a0 = address of string to write
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #20,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        move.l  #2,d2                                                                 ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        lea     string_installation_failed,a0                                         ; a0  : pointer to null-terminated string
        jsr     draw_string_centred_a1        
        ; Red background.        
        move.l  #palette_plane_b_red_red,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        ; TODO Instruct the raccoon to play a sad animation.
        rts
        
installation_screen_installation_success_event: 
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #20,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_installation_complete,a0                                       ; a0  : pointer to null-terminated string
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        ; TODO Instruct the raccoon to play a happy animation.
        ; If we installed a game, show BOOT.
        ; If we installed a firmware, show RESTART.
        ; Show the boot caption and set option availability.
        move.l  installation_screen_should_show_boot_on_success,d0
        beq     installation_screen_installation_success_event_menu_firmware
installation_screen_installation_success_event_game_installation:        
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000
        move.w  #SPRITE_TL+176,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|0),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+48),(MM_VDP_DATA)
        move.w  #SPRITE_TL+160-34,(MM_VDP_DATA)
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #21,d1                                                                ; d1.l: X
        move.l  #23,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_boot,a0                                                        ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        move.l  #CHOICE_AVAILABLE_BOOT,installation_screen_choices_available
        rts       
installation_screen_installation_success_event_menu_firmware:        
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #20,d1                                                                ; d1.l: X
        move.l  #24,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_installation_please_restart_your_system,a0                     ; a0  : pointer to null-terminated string
        lea     draw_string_8x16,a1                                                   ; a1  : string drawing function to use
        jsr     draw_string_centred_a1
        rts        
        
installation_screen_handle_fade_state_fading_in:
        move.l  palette_fade_0_fade_speed,d0
        bne     installation_screen_handle_fade_state_fading_in_not_finished
        move.l  #installation_screen_handle_fade_state_static,installation_screen_fade_state
installation_screen_handle_fade_state_fading_in_not_finished:
        rts
        
installation_screen_handle_fade_state_static:     
        rts
        
installation_screen_handle_fade_state_fading_to_boot:
        move.l  palette_fade_0_fade_speed,d0
        bne     installation_screen_handle_fade_state_fading_to_boot_not_finished
        jmp     game_raccoon_game_partition_boot_sequence
installation_screen_handle_fade_state_fading_to_boot_not_finished:
        rts

; ?!?!?!?!?!
; TODO I want an animated raccoon!
; Magic asynchronous process whose state can be fiddled with by
; the other state handler?
installation_screen_animate_raccoon:
        rts
        
        align   4
installation_failure_caption_table:
        ; This table contains no entry for success (value zero), so subtract 4 bytes when indexing.
        dc.l    string_installation_failure_file_too_big    
        dc.l    string_installation_failure_read_error    
        dc.l    string_installation_failure_verify_error    
        
; - END OF INSTALLATION SCREEN -
; ------------------------------

; browse_screen:
; The browse screen lets the user select a game from the list.
; All of the game data is present in the BOOT partition in the
; directory record cache.
; We can navigate the directory tree stored there,
; each time we do we construct a list of pointers to items
; in that list and then sort them alphabetically just in time.
browse_screen_begin:        
        jsr     plane_b_scrolling_background_initialise
        move.l  #0,plane_b_scrolling_background_status_struct_scroll_speed        
        
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_a
        move.l  #palette_plane_b_lightblue_darkblue,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_a
        move.l  #palette_game_raccoon_logo_blue_font,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_a
        move.l  #palette_game_raccoon_logo_yellow_font,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        
        ; Write the standard pattern to the full 512x256 nametable of Plane B.
        ; I just store the tegels in ROM because I have a lot of it!
        ; 512x256 plane = 64x32 = 2048 tegels
        vram_write VDP_LCOMMAND_VRAM_WRITE,$E000                                      ; Plane B
        move.w  #(2048-1),d0
        lea     intro_screen_plane_b_texture_tegels,a0
browse_screen_begin_load_plane_b_tegels_next_tegel:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d0,browse_screen_begin_load_plane_b_tegels_next_tegel
        
        ; Clear Plane A and the Window Plane, $C000-DFFF
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000                                      ; Plane A
        move.l  #0,d1
        move.l  #((($E000-$C000)/2)-1),d0
browse_screen_begin_clear_plane_a_window_plane_tegels:        
        move.w  d1,(MM_VDP_DATA)
        dbf     d0,browse_screen_begin_clear_plane_a_window_plane_tegels
        
        jsr     plane_b_scrolling_background_update
        
        move.w  #(VDP_REGISTER_WINDOW_PLANE_HORIZONTAL_EXTENT|$00),(MM_VDP_CONTROL)
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|VDP_REGISTER_WINDOW_HUGS_T_EDGE+(4)),(MM_VDP_CONTROL)
                
        ; Set state variables for browse screen:
        move.l  #DIRECTORY_CACHE_BASE,browse_screen_currently_displayed_directory_record_cache_ptr
        jsr     browse_screen_interface_navigate_new_directory        
        ; Write current directory to top part:
        jsr     browse_screen_write_current_directory_caption
        
        ; Write captions to Window plane:                
        ; Lower part has sprites and text, START BOOT, B SELECT, C PARENT
        ; Link to sprite 3 containing hand cursor.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000
        move.w  #SPRITE_TL+190,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|1),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+48),(MM_VDP_DATA)
        move.w  #SPRITE_TL+14,(MM_VDP_DATA)
        move.w  #SPRITE_TL+190,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|2),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+16),(MM_VDP_DATA)
        move.w  #SPRITE_TL+114,(MM_VDP_DATA)
        move.w  #SPRITE_TL+190,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_32|VDP_SPRITE_VSIZE_32|3),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1189+32),(MM_VDP_DATA)
        move.w  #SPRITE_TL+219,(MM_VDP_DATA)
        ; Sprite 3 links to 0 and has no data yet in case it gets drawn.
        move.w  #0,(MM_VDP_DATA)
        move.w  #0,(MM_VDP_DATA)
        move.l  #1,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #7,d1                                                                 ; d1.l: X
        move.l  #25,d2                                                                ; d2.l: Y
        move.l  #1,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        lea     string_boot,a0                                                        ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        move.l  #19,d1                                                                ; d1.l: X
        lea     string_select,a0                                                      ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8
        move.l  #32,d1                                                                ; d1.l: X
        lea     string_parent,a0                                                      ; a0  : pointer to null-terminated string
        jsr     draw_string_8x8                

        move.l  #browse_screen_handle_fade_state_fading_in,browse_screen_fade_state       
        
browse_screen_loop:                 
        ; Middle screen trigger enables Window display on lower six rows of screen.        
        wait_for_vcounter_equals 112                      
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|VDP_REGISTER_WINDOW_HUGS_B_EDGE+(28-6)),(MM_VDP_CONTROL)

        lea     pad_state,a0
        jsr     controllers_read_and_derive
        jsr     palette_fade_struct_update                
        
        jsr     browse_screen_position_raccoon_hand_sprite_arrow_widgets
        
        jsr     browse_screen_interface_update_plane_a_scroll_register
        
        move.l  browse_screen_fade_state,a0
        jsr     (a0)

        ; Lower screen (during VBlank) trigger enables Window display on upper four rows of screen.
        wait_for_vcounter_equals (224)
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|VDP_REGISTER_WINDOW_HUGS_T_EDGE+(4)),(MM_VDP_CONTROL)
        
        jmp     browse_screen_loop
        
; browse_screen_position_raccoon_hand_sprite_arrow_widgets:
; Place the raccoon hand sprite and the arrow widgets if they are visible.
; Invalidates all registers.        
browse_screen_position_raccoon_hand_sprite_arrow_widgets:
        ; Raccoon selection hand sprite, TODO maybe make this animated?
        ; Calculate the on-screen position of the cursor:
        ; 4*8 + browse_screen_cursor_current_position_pixels - browse_screen_scroll_current_position_pixels
        move.l  browse_screen_cursor_current_position_pixels,d0
        sub.l   browse_screen_scroll_current_position_pixels,d0
        addi.l  #(4*8 + SPRITE_TL),d0        
        ; We're modifying sprite slots 3 (raccoon hand) -> 4 (up) and -> 5 (down) which links to zero.
        vram_write VDP_LCOMMAND_VRAM_WRITE,$F000+8*3
        move.w  d0,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_16|VDP_SPRITE_VSIZE_16|4),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1177+0),(MM_VDP_DATA)
        move.w  #SPRITE_TL+0,(MM_VDP_DATA)
        
        ; Default X position of arrow widgets, common to both.
        move.w  #SPRITE_TL+287,d2
        ; Show 'up' conditionally, change the widgets visibility by modifying their X position (stored in d0).
        move.w  d2,d0
        ; Up is shown if the scroll destination is nonzero.
        move.l  browse_screen_scroll_destination_position_pixels,d1
        bne     browse_screen_position_raccoon_hand_sprite_arrow_widgets_up_widget_visible
        move.w  #10,d0                                                                ; Set X to offscreen position (not special zero though)
browse_screen_position_raccoon_hand_sprite_arrow_widgets_up_widget_visible:        
        move.w  #SPRITE_TL+16,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_16|VDP_SPRITE_VSIZE_16|5),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1181+0),(MM_VDP_DATA)
        move.w  d0,(MM_VDP_DATA)
                
        ; Down is shown if the scroll destination is < maximum possible.        
        ; Calculate maximum possible scroll value.
        ; Max scroll value in d1 = 16*max(0,(no_items - list_visual_max_size))
        move.l  browse_screen_no_items,d1
        sub.l   #BROWSE_SCREEN_INTERFACE_NO_VISIBLE_ROWS,d1
        move.l  #0,d0
        jsr     max
        lsl.l   #4,d0
        move.l  d0,d1        
        ; Show 'down' conditionally, change the widgets visibility by modifying their X position (stored in d0).
        move.w  d2,d0
        ; If scroll_dest d3 < max_scroll_dest d1 show...
        move.l  browse_screen_scroll_destination_position_pixels,d3
        cmp.l   d1,d3
        blt     browse_screen_position_raccoon_hand_sprite_arrow_widgets_down_widget_visible
        move.w  #10,d0                                                                ; Set X to offscreen position (not special zero though)
browse_screen_position_raccoon_hand_sprite_arrow_widgets_down_widget_visible:        
        move.w  #SPRITE_TL+165,(MM_VDP_DATA)
        move.w  #(VDP_SPRITE_HSIZE_16|VDP_SPRITE_VSIZE_16|0),(MM_VDP_DATA)
        move.w  #(VDP_PALETTE_1|1185+0),(MM_VDP_DATA)
        move.w  d0,(MM_VDP_DATA)        
        
        rts        
        
browse_screen_handle_fade_state_fading_in:
        move.l  palette_fade_0_fade_speed,d0
        bne     browse_screen_handle_fade_state_fading_in_not_finished
        move.l  #browse_screen_handle_fade_state_static,browse_screen_fade_state
browse_screen_handle_fade_state_fading_in_not_finished:
        rts
        
browse_screen_handle_fade_state_static:     
        ; Handle scrolling and cursor motion and the sprite.
        jsr     browse_screen_handle_scrolling
        jsr     browse_screen_handle_cursor_scrolling
        
        ; Detect input and branch.
        jsr     browse_screen_handle_input
        
        rts
        
browse_screen_handle_fade_state_fading_to_boot:
        move.l  palette_fade_0_fade_speed,d0
        bne     browse_screen_handle_fade_state_fading_to_boot_not_finished
        jmp     game_raccoon_game_partition_boot_sequence
browse_screen_handle_fade_state_fading_to_boot_not_finished:
        rts

BROWSE_SCREEN_SCROLL_SPEED = 4
        
; browse_screen_handle_scrolling:
browse_screen_handle_scrolling:
        ; If the destination is not the same as the current
        ; modify the current to approach it.
        ; Move in multiples of two?
        move.l  browse_screen_scroll_destination_position_pixels,d0
        cmp.l   browse_screen_scroll_current_position_pixels,d0
        bgt     browse_screen_handle_scrolling_need_to_increase
        blt     browse_screen_handle_scrolling_need_to_decrease
        rts
browse_screen_handle_scrolling_need_to_increase:        
        addi.l  #BROWSE_SCREEN_SCROLL_SPEED,browse_screen_scroll_current_position_pixels
        rts
browse_screen_handle_scrolling_need_to_decrease:        
        subi.l  #BROWSE_SCREEN_SCROLL_SPEED,browse_screen_scroll_current_position_pixels                
        rts
        
; browse_screen_handle_cursor_scrolling:
browse_screen_handle_cursor_scrolling:
        ; If the destination is not the same as the current
        ; modify the current to approach it.
        ; Move in multiples of two?
        move.l  browse_screen_cursor_destination_position_pixels,d0
        cmp.l   browse_screen_cursor_current_position_pixels,d0
        bgt     browse_screen_handle_cursor_scrolling_need_to_increase
        blt     browse_screen_handle_cursor_scrolling_need_to_decrease
        rts
browse_screen_handle_cursor_scrolling_need_to_increase:        
        addi.l  #BROWSE_SCREEN_SCROLL_SPEED,browse_screen_cursor_current_position_pixels
        rts
browse_screen_handle_cursor_scrolling_need_to_decrease:        
        subi.l  #BROWSE_SCREEN_SCROLL_SPEED,browse_screen_cursor_current_position_pixels
        rts
        
; browse_screen_handle_input:
; Detect input and branch.
; Invalidates all registers.
browse_screen_handle_input: 
        ; Up, Down, Left, Right, Start, B and C. Seven buttons.
        ; Let's do it to it.
        
        ; Only allow input if no scrolling is happening.
        move.l  browse_screen_cursor_destination_position_pixels,d0
        cmp.l   browse_screen_cursor_current_position_pixels,d0
        beq     browse_screen_handle_input_proceed
        rts
browse_screen_handle_input_proceed:       
        ; We might jmp out to a handler.
        ; These handlers contain their own rts to the caller of 
        ; browse_screen_handle_input, so only one handler may execute.        
        
        ; Start press, C press.
        btst    #CONTROLLER_S_BIT_btst_l,pad_state+PAD_1+PRESSED+LOW_BYTE
        bne     browse_screen_handle_input_not_start
        jmp     browse_screen_handle_input_start
browse_screen_handle_input_not_start:
        btst    #CONTROLLER_C_BIT_btst_l,pad_state+PAD_1+PRESSED+LOW_BYTE
        bne     browse_screen_handle_input_not_c
        jmp     browse_screen_handle_input_c
browse_screen_handle_input_not_c:
                
        ; Only respond to any of the following if the current directory
        ; is not empty.        
        move.l  browse_screen_no_items,d0
        bne     browse_screen_handle_input_selection_may_proceed
        rts
browse_screen_handle_input_selection_may_proceed:       
        ; B press.                
        btst    #CONTROLLER_B_BIT_btst_l,pad_state+PAD_1+PRESSED+LOW_BYTE
        bne     browse_screen_handle_input_not_b
        jmp     browse_screen_handle_input_b
browse_screen_handle_input_not_b:
        ; Left press, right press.        
        btst    #CONTROLLER_L_BIT_btst_l,pad_state+PAD_1+PRESSED+LOW_BYTE
        bne     browse_screen_handle_input_not_left
        jmp     browse_screen_handle_input_left
browse_screen_handle_input_not_left:
        btst    #CONTROLLER_R_BIT_btst_l,pad_state+PAD_1+PRESSED+LOW_BYTE
        bne     browse_screen_handle_input_not_right
        jmp     browse_screen_handle_input_right
browse_screen_handle_input_not_right:
        ; Up curr, down curr.
        btst    #CONTROLLER_U_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     browse_screen_handle_input_not_up
        jmp     browse_screen_handle_input_up
browse_screen_handle_input_not_up:
        btst    #CONTROLLER_D_BIT_btst_l,pad_state+PAD_1+CURR+LOW_BYTE
        bne     browse_screen_handle_input_not_down
        jmp     browse_screen_handle_input_down
browse_screen_handle_input_not_down: 
        rts
        
browse_screen_handle_input_start:
        ; Transition to boot.
        move.l  #palette_black,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_black,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_black,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed
        move.l  #browse_screen_handle_state_fading_to_boot,browse_screen_fade_state
        rts        
        
browse_screen_handle_input_b:
        ; B has been pressed and there are definitely files in this directory.
           
        ; Retrieve the pointer behind item at cursor in the sorted list into d0:
        ; Dereference directory_cache_items_list_begin + 4*cursor
        move.l  browse_screen_selected_item_in_list_index,d0     
        move.l  d0,d1                                                                 ; Copy into d1 for comparing against number of directory entries in a moment.
        lsl.l   #2,d0
        add.l   #directory_cache_items_list_begin,d0
        move.l  d0,a0
        move.l  (a0),d0
        ; Do something different with d0 depending on whether it's a file or a directory.
        cmp.l   browse_screen_no_directories,d1
        blt     browse_screen_handle_input_b_enter_directory
browse_screen_handle_input_b_select_file:
        ; Retrieve the size of the file in sectors for installation screen.
        move.l  d0,a0
        move.l  #0,d1
        move.w  DIRECTORY_CACHE_OFFSET_TO_FILE_SIZE_IN_SECTORS(a0),d1
        move.l  d1,installation_screen_file_size_in_sectors
        ; Move to install the selected file. What file is that?
        ; Get the pointer in d0, strip the directory base and shift back into 
        ; an absolute index in the rom list.     
        andi.l  #DIRECTORY_CACHE_INDEX_MASK,d0
        lsr.l   #8,d0
        move.w  d0,installation_screen_chosen_word
        move.l  #1,installation_screen_should_show_boot_on_success
        
        move.l  #palette_solid_lightblue_transition,palette_fade_0_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_0_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_1_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_1_fade_speed
        move.l  #palette_solid_lightblue_transition,palette_fade_2_ptr_b
        move.l  #PALETTE_FADE_SPEED_FAST,palette_fade_2_fade_speed        
        move.l  #browse_screen_handle_state_fading_to_installation,browse_screen_fade_state        
        rts
                
browse_screen_handle_input_b_enter_directory:
        ; d0 contains the address in ROM of the directory we're jumping to.
        move.l  d0,browse_screen_currently_displayed_directory_record_cache_ptr
        jsr     browse_screen_interface_navigate_new_directory        
        ; Write current directory to top part:
        jsr     browse_screen_write_current_directory_caption        
        rts
        
browse_screen_handle_state_fading_to_boot:
        move.l  palette_fade_0_fade_speed,d0
        bne     browse_screen_handle_state_fading_to_boot_not_finished
        ; Disable window plane.
        move.w  #(VDP_REGISTER_WINDOW_PLANE_HORIZONTAL_EXTENT|$00),(MM_VDP_CONTROL)
        move.w  #(VDP_REGISTER_WINDOW_PLANE_VERTICAL_EXTENT|$00),(MM_VDP_CONTROL)
        jmp     game_raccoon_game_partition_boot_sequence
browse_screen_handle_state_fading_to_boot_not_finished:
        rts

browse_screen_handle_state_fading_to_installation:
        move.l  palette_fade_0_fade_speed,d0
        bne     browse_screen_handle_state_fading_to_installation_not_finished
        jmp     installation_screen_begin
browse_screen_handle_state_fading_to_installation_not_finished:
        rts                
        
browse_screen_handle_input_c:   
        ; Abort if browse_screen_currently_displayed_directory_record_cache_ptr = DIRECTORY_CACHE_BASE

        ; We're moving to the folder whose base pointer can be found at
        ; = $200000 + $100*browse_screen_currently_displayed_directory_record_cache_ptr's parent field
        move.l  browse_screen_currently_displayed_directory_record_cache_ptr,d0
        cmp.l   #DIRECTORY_CACHE_BASE,d0
        bne     browse_screen_handle_input_c_proceed
        rts
browse_screen_handle_input_c_proceed:   
        move.l  d0,a0     
        move.l  #0,d0
        move.w  DIRECTORY_CACHE_OFFSET_TO_PARENT_INDEX(a0),d0
        lsl.l   #8,d0
        ori.l   #DIRECTORY_CACHE_BASE,d0
        move.l  d0,browse_screen_currently_displayed_directory_record_cache_ptr
        jsr     browse_screen_interface_navigate_new_directory        
        ; Write current directory to top part:
        jsr     browse_screen_write_current_directory_caption
        rts     
        
        ; These were suggested to allow the user to change selection location
        ; by whole pages, but haven't put it in.
browse_screen_handle_input_left:
        rts
        
browse_screen_handle_input_right:
        rts
        
browse_screen_handle_input_up:
        ; Abort if already zero.
        move.l  browse_screen_selected_item_in_list_index,d0
        bgt     browse_screen_handle_input_up_proceed
        rts
browse_screen_handle_input_up_proceed:
        subi.l  #1,browse_screen_selected_item_in_list_index
        subi.l  #16,browse_screen_cursor_destination_position_pixels
        
        ; If scroll_dest <= 0 abort
        move.l  browse_screen_scroll_destination_position_pixels,d0
        ble     browse_screen_handle_input_up_proceed_no_scroll_effect
        ; If cursor_dest < (scroll_dest + 15), act
        add.l   #15,d0
        cmp.l   browse_screen_cursor_destination_position_pixels,d0
        blt     browse_screen_handle_input_up_proceed_no_scroll_effect
        subi.l  #16,browse_screen_scroll_destination_position_pixels
        ; Write the row that's being scrolled to.
        move.l  browse_screen_scroll_destination_position_pixels,d0
        lsr.l   #4,d0
        jsr     browse_screen_interface_draw_text_row_d0
browse_screen_handle_input_up_proceed_no_scroll_effect:        
        rts
        
browse_screen_handle_input_down:    
        ; Abort if already on final item in the list.
        move.l  browse_screen_selected_item_in_list_index,d0
        addi.l  #1,d0
        cmp.l   browse_screen_no_items,d0
        blt     browse_screen_handle_input_down_proceed
        rts
browse_screen_handle_input_down_proceed:
        addi.l  #1,browse_screen_selected_item_in_list_index
        addi.l  #16,browse_screen_cursor_destination_position_pixels
        
        ; Calculate maximum possible scroll value.
        ; Max scroll value in d1 = 16*max(0,(no_items - list_visual_max_size))
        move.l  browse_screen_no_items,d1
        sub.l   #BROWSE_SCREEN_INTERFACE_NO_VISIBLE_ROWS,d1
        move.l  #0,d0
        jsr     max
        lsl.l   #4,d0
        move.l  d0,d1
        
        ; If scroll_dest d0 >= max_scroll_dest d1 abort
        move.l  browse_screen_scroll_destination_position_pixels,d0
        cmp.l   d1,d0
        bge     browse_screen_handle_input_down_proceed_no_scroll_effect
        ; If cursor_dest memory >= ((visual_size - 1) + scroll_dest) d0, act
        add.l   #(BROWSE_SCREEN_INTERFACE_NO_VISIBLE_ROWS-2)*16,d0
        cmp.l   browse_screen_cursor_destination_position_pixels,d0
        bge     browse_screen_handle_input_down_proceed_no_scroll_effect
        addi.l  #16,browse_screen_scroll_destination_position_pixels
        ; Write the row that's being scrolled to.
        move.l  browse_screen_scroll_destination_position_pixels,d0
        lsr.l   #4,d0
        add.l   #(BROWSE_SCREEN_INTERFACE_NO_VISIBLE_ROWS-1),d0
        jsr     browse_screen_interface_draw_text_row_d0
browse_screen_handle_input_down_proceed_no_scroll_effect:        
        rts
           
        
; browse_screen_write_current_directory_caption:        
; Write the name of the current directory (the name of the
; parent directory), but truncated to show only the rightmost
; 40-4 characters (so it doesn't exceed the window), but
; shows any precending directories leading up to the current.
; Invalidates all registers.        
browse_screen_write_current_directory_caption:  
        jsr     browse_screen_clear_current_directory_caption      
        move.l  browse_screen_currently_displayed_directory_record_cache_ptr,a0
        jsr     string_length
        ; If the length of the string is greater than 36, then advance a0
        ; by how many cells beyond 36 it is.
        sub.l   #36,d0
        blt     browse_screen_write_current_directory_caption_ready
        lea     (a0,d0),a0
browse_screen_write_current_directory_caption_ready:        
        move.l  #1,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #2,d1                                                                 ; d1.l: X
        move.l  #2,d2                                                                 ; d2.l: Y
        move.l  #2,d3                                                                 ; d3.l: palette line
        move.l  #64,d4                                                                ; d4.l: width of plane to write into for clipping = 64
        jsr     draw_string_8x8
        rts

; browse_screen_clear_current_directory_caption:
; Erase the caption from the Window plane: 36 tegels.        
browse_screen_clear_current_directory_caption:     
        vram_write VDP_LCOMMAND_VRAM_WRITE,$D000+(2*(2+(2*64)))
        move.w  #0,d1
        move.w  #(36-1),d0
browse_screen_clear_current_directory_caption_next_tegel:
        move.w  #0,(MM_VDP_DATA)        
        dbf     d0,browse_screen_clear_current_directory_caption_next_tegel
        rts

; browse_screen_interface_force_redraw_new_directory:
; The user has navigated to a new directory.
; The directory ptr is stored in
; browse_screen_currently_displayed_directory_record_cache_ptr
; Clear the visible list and reset all the scrolls.        
; No registers invalidated.
browse_screen_interface_navigate_new_directory:        
        movem.l d0-d7/a0-a6,-(a7)
        
        ; Clear Plane A, $C000-CFFF
        vram_write VDP_LCOMMAND_VRAM_WRITE,$C000                                      ; Plane A
        move.l  #0,d1
        move.l  #((($D000-$C000)/2)-1),d0
browse_screen_handle_input_b_enter_directory_clear_plane_a_window_plane_tegels:        
        move.w  d1,(MM_VDP_DATA)
        dbf     d0,browse_screen_handle_input_b_enter_directory_clear_plane_a_window_plane_tegels
        
        move.l  browse_screen_currently_displayed_directory_record_cache_ptr,a0
        
        move.l  #0,browse_screen_selected_item_in_list_index
        
        move.l  #0,browse_screen_cursor_current_position_pixels
        move.l  #0,browse_screen_cursor_destination_position_pixels
        move.l  #0,browse_screen_scroll_current_position_pixels
        move.l  #0,browse_screen_scroll_destination_position_pixels
        
        ; Move raccoon hand to top during sorting/reading.
        jsr     browse_screen_position_raccoon_hand_sprite_arrow_widgets
        
        ; Set the scroll value of Plane A to reflect the scroll position.
        jsr     browse_screen_interface_update_plane_a_scroll_register
        
        ; Read from the directory record cache the statistics for the selected directory.
        move.l  #0,d1
        move.l  d1,d2
        move.w  DIRECTORY_CACHE_OFFSET_TO_DIRECTORY_COUNT(a0),d1
        move.l  d1,browse_screen_no_directories
        move.w  DIRECTORY_CACHE_OFFSET_TO_FILE_COUNT(a0),d2
        move.l  d2,browse_screen_no_files
        move.l  d2,d3
        add.w   d1,d3
        move.l  d3,browse_screen_no_items
        
        ; If there are no items in this directory, engage THE MODE.
        beq     browse_screen_interface_force_redraw_new_directory_is_empty        
        ; If the current directory is not empty,
        ; - Copy all the directories to the directory cache items list then sort them.
        ; - Then copy all the files to the directory cache items list then sort them.
        ; Hey, that seems like a pattern!
        ; a0 needs to point to the contents of this directory first.
        ; = $200000 + $100*current_dir_contents
        move.l  #0,d0
        move.w  DIRECTORY_CACHE_OFFSET_TO_CONTENTS_BEGIN(a0),d0
        lsl.l   #8,d0
        ori.l   #DIRECTORY_CACHE_BASE,d0
        move.l  d0,a0
        ; Now load items based at this a0.
        lea     directory_cache_items_list_begin,a1
        move.l  d1,d0
        jsr     directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1
        ; Advance a1 by d1*4
        ; Advance a0 by d1*256
        lsl.l   #2,d1
        lea     (a1,d1),a1
        lsl.l   #6,d1
        lea     (a0,d1),a0
        ; Now a0 points to the first directory record for files.
        ; a1 points to the next vacant directory_cache_items_list_begin entry.
        ; d0 needs to contain number of files.
        move.l  d2,d0
        jsr     directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1
        
        ; Draw the list of items up to the limit of the screen
        jsr     browse_screen_interface_redraw_full_screen
        
        movem.l (a7)+,d0-d7/a0-a6
        rts          
browse_screen_interface_force_redraw_new_directory_is_empty:
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #2,d1                                                                 ; d1.l: X
        move.l  #4,d2                                                                 ; d2.l: Y calculated above
        move.l  #2,d3                                                                 ; d3.l: Palette line calculated above
        move.l  #38,d4                                                                ; d4.l: Clip drawing.
        lea     string_directory_is_empty,a0
        jsr     draw_string_8x16
        
        movem.l (a7)+,d0-d7/a0-a6
        rts  
        
browse_screen_interface_redraw_full_screen:
        movem.l d0-d2,-(a7)
        
        ; Draw at most BROWSE_SCREEN_INTERFACE_NO_VISIBLE_ROWS items
        ; Draw at most browse_screen_no_items-(browse_screen_cursor_current_position_pixels/16)
        move.l  browse_screen_no_items,d1
        move.l  browse_screen_cursor_current_position_pixels,d0
        lsr.l   #4,d0
        move.l  d0,d2                                                                 ; Copy index of topmost item into d2 for later.
        sub.l   d0,d1
        move.l  #BROWSE_SCREEN_INTERFACE_NO_VISIBLE_ROWS,d0
        jsr     min                                                                   ; d0 contains number of rows to draw.        
        sub.l   #1,d0                                                                 ; Make it a dbf-compatible loop counter.
        ; Move loop counter into d1 and index of topmost row into d0.
        move.l  d0,d1
        move.l  d2,d0
        
        ; Draw items from index d0 upwards until we run out of d1.
browse_screen_interface_redraw_full_screen_next_line:
        jsr     browse_screen_interface_draw_text_row_d0
        add.l   #1,d0
        dbf     d1,browse_screen_interface_redraw_full_screen_next_line       
                        
        movem.l (a7)+,d0-d2
        rts                
        
; browse_screen_interface_draw_text_row_d0:
; d0.l contains the index of a row to draw within the current
; directory. 
; Draw the row to Plane A using absolute coordinates: we rely
; on the scroll registers and window stuff to have this appear
; in the correct location on screen.
; Draw row 0 at 0 and then 2x every row after.
; Loop if it would go off the bottom of the plane.
browse_screen_interface_draw_text_row_d0:   
        movem.l d0-d7/a0,-(a7)

        ; Convert d0 into an index into directory_cache_items_list_begin,
        ; then make the complete pointer, then dereference to get the
        ; string in ROM behind this sorted list index.
        move.l  d0,d1
        lsl.l   #2,d1
        lea     directory_cache_items_list_begin,a0
        lea     (a0,d1),a0
        move.l  (a0),a0
        
        ; Calculate tile row within Plane A to draw at into d2. (row*2) mod 32
        move.l  d0,d2
        lsl.l   #1,d2
        and.l   #31,d2
        
        ; Determine palette colour: 2 if index < no_directories, else 1.
        move.l  #1,d3
        cmp.l   browse_screen_no_directories,d0
        bge     browse_screen_interface_draw_text_row_d0_not_directory_colour
        add.l   #1,d3
browse_screen_interface_draw_text_row_d0_not_directory_colour:  
        
        ; Clear two rows of tegels on this row. = Plane A base + 2*64*row
        move.l  d2,d0
        lsl.l   #7,d0
        add.l   #$C000,d0
        jsr     vram_write_swizzle_address_in_d0
        move.l  d0,(MM_VDP_CONTROL)
        move.l  #((64*2)-1),d4
browse_screen_interface_draw_text_row_d0_clear_next_tegel:        
        move.w  #0,(MM_VDP_DATA)
        dbf     d4,browse_screen_interface_draw_text_row_d0_clear_next_tegel
        
        move.l  #0,d0                                                                 ; d0.l: plane (0=A, 1=W, 2=B)
        move.l  #2,d1                                                                 ; d1.l: X
                                                                                      ; d2.l: Y calculated above
                                                                                      ; d3.l: Palette line calculated above
        move.l  #38,d4                                                                ; d4.l: Clip drawing.

        jsr     string_pointer_from_final_slash
        jsr     draw_string_8x16
       
        movem.l (a7)+,d0-d7/a0
        rts
        
; browse_screen_interface_update_plane_a_scroll_register:
; Set the scroll value of Plane A to reflect the scroll position.
; No registers invalidated.
browse_screen_interface_update_plane_a_scroll_register:
        movem.l d0,-(a7)
        
        vram_write (VDP_LCOMMAND_VSRAM_WRITE),$0000+0
        move.l  #(-(8*4)),d0
        add.l   browse_screen_scroll_current_position_pixels,d0
        move.w  d0,(MM_VDP_DATA)
        
        movem.l (a7)+,d0
        rts
        
; directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1:        
; d0.l contains the number of items (directories or files) to load
; and store.
; a0 is a pointer to the first of these items to be read from the
; directory record cache in ROM.
; a1 is a pointer to directory_cache_items_list_begin+4*X to store
; the sorted pointers in.
; No registers invalidated.
directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1:     
        move.l  d0,d0
        beq     directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1_done        
        movem.l d0-d1/a0-a2,-(a7)
        
        ; d1 keeps track of number of items we want to read and sort.
        ; a2 keeps track of original base pointer into list we want to sort.
        move.l  d0,d1 
        move.l  a1,a2
        
        ; Write all of the pointers of the items we want to sort into the a1 destination list.
        sub.l   #1,d0
directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1_write_next_unsorted_pointer:        
        move.l  a0,(a1)+
        lea     DIRECTORY_CACHE_ENTRY_SIZE_BYTES(a0),a0
        dbf     d0,directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1_write_next_unsorted_pointer
                
        ; Sort the list of d1 items based at a2.
        jsr     directory_cache_items_list_sort_address_list_alphabetically
                        
        movem.l (a7)+,d0-d1/a0-a2
directory_cache_items_list_read_d0_items_from_a0_sort_store_in_a1_done:
        rts         

; directory_cache_items_list_sort_address_list_alphabetically:
; From a2 for d1.w number of longwords, sort the directory cache records by their filenames.
; No return - sort the list in place. IF YOU DARE.
; No registers invalidated.
directory_cache_items_list_sort_address_list_alphabetically:
        movem.l d0-d7/a0-a6,-(a7)
        
directory_cache_items_list_sort_address_list_alphabetically_next_iteration:     
        cmp.w   #2,d1                                                                 ; If there is one or zero items, then return.
        bge     directory_cache_items_list_sort_address_list_alphabetically_continue_iteration
        movem.l (a7)+,d0-d7/a0-a6
        rts       
directory_cache_items_list_sort_address_list_alphabetically_continue_iteration:
        ; Alright, we're going to find the 'least' item as we go through d1 items.
        ; After finding the 'least' item, swap its value with the one in a2 and
        ; decrement d1 and increment a2, since we have placed the least item in
        ; the appropriate location.
        ; Then repeat to sort the list until there are no more items to sort.
        ; a3 is a sliding cursor going from a2+4 onwards for d1-2 advances.
        move.w  d1,d0                                                                 ; Copy d1 item list counter in d0 use as a loop counter.
        sub.w   #2,d0                                                                 ; d0 loops through number of items in original d1 list minus 1 (-2 because it's a dbf loop).
        move.l  a2,a3                                                                 ; a3 selects the 'right hand item' to compare against.
directory_cache_items_list_sort_address_list_alphabetically_advance_sliding_cursor:        
        lea     4(a3),a3                                                              ; Move a3 one cell to the right.
        move.l  (a2),a0                                                               ; Retrieve the contents of the left cell.
        move.l  (a3),a1                                                               ; Retrieve the contents of the right cell.
        jsr     strcmp                                                                ; CC equals a0 <?> a1. TODO may be a lot faster to inline a version using unused a registers.
        bge     directory_cache_items_list_sort_address_list_alphabetically_no_swap   ; Do nothing if the left cell is already less than the right cell.
        move.l  a0,(a3)                                                               ; Swap the contents of (a3) and (a2)
        move.l  a1,(a2)                                                               ; Swap ^        
directory_cache_items_list_sort_address_list_alphabetically_swap_complete:   
directory_cache_items_list_sort_address_list_alphabetically_no_swap:
        dbf     d0,directory_cache_items_list_sort_address_list_alphabetically_advance_sliding_cursor
        
        sub.w   #1,d1                                                                 ; The length of the unsorted list is one shorter since we have sorted one of the items.
        lea     4(a2),a2                                                              ; The address of the unsorted part of the list is one beyond what it was.
        jmp     directory_cache_items_list_sort_address_list_alphabetically_next_iteration             

; max:
; Return in d0.l the max of d0.l and d1.l
; Only d0 changed.
max:
        cmp.l   d0,d1
        ble     max_return
        move.l  d1,d0
max_return:        
        rts
; min:
; Return in d0.l the min of d0.l and d1.l
; Only d0 changed.
min:
        cmp.l   d0,d1
        bge     min_return
        move.l  d1,d0
min_return:        
        rts
        
BROWSE_SCREEN_INTERFACE_NO_VISIBLE_ROWS = 9        
        
DIRECTORY_CACHE_BASE                           = $200000
DIRECTORY_CACHE_INDEX_MASK                     = $1FFFFF
DIRECTORY_CACHE_ENTRY_SIZE_BYTES               =    $100       
DIRECTORY_CACHE_ENTRY_SIZE_WORDS               = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES/2)
; Directory type records.
DIRECTORY_CACHE_OFFSET_TO_DIRECTORY_COUNT      = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(4*2))
DIRECTORY_CACHE_OFFSET_TO_FILE_COUNT           = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(3*2))
DIRECTORY_CACHE_OFFSET_TO_CONTENTS_BEGIN       = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(2*2))
DIRECTORY_CACHE_OFFSET_TO_PARENT_INDEX         = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(1*2))
; File type records.
DIRECTORY_CACHE_OFFSET_TO_FILE_SIZE_IN_SECTORS = (DIRECTORY_CACHE_ENTRY_SIZE_BYTES-(1*2))
                
; string_pointer_from_final_slash:
; Given a null-terminated string pointer in a0,
; advance it to the end of the string, then rewind
; until a forward slash is hit, then leave in a0
; a pointer to the first character after the final
; slash in the string.
; If there is no slash in the string, leave a0
; unchanged.
; Only changes a0.
string_pointer_from_final_slash:
        move.l  a1,-(a7)
        
        move.l  a0,a1
string_pointer_from_final_slash_looking_for_end:        
        cmpi.b  #0,(a0)+
        bne     string_pointer_from_final_slash_looking_for_end
        ; a0 is now one char beyond the null terminator.
string_pointer_from_final_slash_rewind:
        cmp.l   a0,a1
        beq     string_pointer_from_final_slash_no_slash                              ; If the rewind has made a0 equal its original value, return it.
        cmpi.b  #'/',-(a0)
        bne     string_pointer_from_final_slash_rewind
        lea     1(a0),a0                                                              ; Advance the cursor back after the slash again.        
string_pointer_from_final_slash_no_slash:

        move.l  (a7)+,a1
        rts
        
; strcmp:        
; Set the condition code to indicate whether the string pointed to by a0 is
; greater or less than the one in a1.
; Compare the characters in both and advance.
; If they're different, then return and the result is in the condition code.
; If they're the same, check for zero. If they're the same and zero, they're equal, return that in the condition code.
; Otherwise they're the same but nonzero, so repeat the test.
; No registers invalidated.
strcmp:
        movem.l a0/a1,-(a7)
strcmp_next_char:
        cmpm.b  (a0)+,(a1)+
        bne     strcmp_return        
        cmpi.b  #0,-1(a0)
        bne     strcmp_next_char        
strcmp_return:
        movem.l (a7)+,a0/a1
        rts        

; - END OF BROWSE SCREEN -
; ------------------------

; Given palette pointers in a0 and a1,
; write an interpolated palette (level given in d1, a0 -0--256-> a1)
; to the given palette line d0.l in CRAM.
; If a0 ==  a1, then a0 palette is just copied.
; If d1 ==   0, a0 is copied.
; If d1 => 256, a1 is copied.
; No registers invalidated.
palette_write_interpolated_palette:
        ; Try early outs:
        move.w  d1,d1
        beq     palette_write_palette
        cmp.w   #256,d1
        blt     palette_write_interpolated_palette_not_a1
        movem.l a0,-(a7)
        move.l  a1,a0                                                                 ; Write just the end palette, move a1 to a0
        jsr     palette_write_palette
        movem.l (a7)+,a0
        rts
palette_write_interpolated_palette_not_a1:
        
        movem.l d0-d7/a0-a6,-(a7)
        
        move.w  #21,d2                                                                ; Bit number of A5 in VDP command address.
        lsl.l   d2,d0                                                                 ; d0 contains VDP-command-compatible address of palette line.
        or.l    #VDP_LCOMMAND_CRAM_WRITE,d0                                           ; Construct VDP command.
        move.l  d0,(MM_VDP_CONTROL)                                                   ; Send command, d0 and d2 available for use.
        
        move.w  d1,d2                                                                 ; d2 = a1 factor
        move.w  #256,d3
        sub.w   d2,d3                                                                 ; d2 = a1 factor, d3 = a0 factor
        
        move.w  #(PALETTE_SIZE_WORD_ENTRIES-1),d7
palette_write_interpolated_palette_next_entry:
        ; Compose RGB ((a0)+ * (256-d1)) + ((a1)+ * d1) into d0.w.                
        move.w  (a0)+,d0                                                              ; d0 = source colour
        move.w  d3,d1                                                                 ; d0 = source colour, d1 = fade factor
        jsr     colour_fade                                                           ; d0 = source colour * fade factor
        move.w  d0,d4                                                                 ; d4 = source colour * fade factor
        
        move.w  (a1)+,d0                                                              ; d0 = dest colour
        move.w  d2,d1                                                                 ; d0 = dest colour, d1 = fade factor
        jsr     colour_fade                                                           ; d0 = dest colour * fade factor
        
        add.w   d4,d0                                                                 ; d0 = final colour        
        move.w  d0,(MM_VDP_DATA)                                                      ; Write it.
        dbf     d7,palette_write_interpolated_palette_next_entry
        
        movem.l (a7)+,d0-d7/a0-a6
        
        rts

; colour_fade:        
; Given a Mega Drive colour value $0BGR in d0.w, and a fixed point fade factor F in d1.w (1-255),
; return a new colour in d0.w = (BGR * F)/256
; Invalidates no registers.
colour_fade:
        movem.l d2-d3,-(a7)
                
        move.w  d0,d2                                                                 ; d2 = $0BGR
        and.w   #$000E,d2                                                             ; d2 = $000R
        mulu.w  d1,d2                                                                 ; d2 = $000R * F in .8 fixed
        and.w   #$0E00,d2                                                             ; d3 = $0R00
        lsr.w   #8,d2                                                                 ; d2 = $000R * F
                
        move.w  d0,d3                                                                 ; d3 = $0BGR
        and.w   #$00E0,d3                                                             ; d3 = $00G0
        lsr.w   #4,d3                                                                 ; d3 = $000G
        mulu.w  d1,d3                                                                 ; d3 = $000G * F in .8 fixed
        and.w   #$0E00,d3                                                             ; d3 = $0G00 * F
        lsr.w   #4,d3                                                                 ; d3 = $00G0 * F
                
        lsr.w   #8,d0                                                                 ; d0 = $000B
        mulu.w  d1,d0                                                                 ; d0 = $000B * F in .8 fixed
        and.w   #$0E00,d0                                                             ; d0 = $0B00 * F
        or.w    d3,d0                                                                 ; d0 = $0BG0 * F
        or.w    d2,d0                                                                 ; d0 = $0BGR * F
        
        movem.l (a7)+,d2-d3
        rts        
        
; Given palette pointer in a0,
; Write palette to the given palette line d0.l in CRAM.
; No registers invalidated.
palette_write_palette:
        movem.l d0/d1/a0,-(a7)
        
        move.w  #21,d1                                                                ; Bit number of A5 in VDP command address.
        lsl.l   d1,d0                                                                 ; d0 contains VDP-command-compatible address of palette line.
        or.l    #VDP_LCOMMAND_CRAM_WRITE,d0                                           ; Construct VDP command.
        move.l  d0,(MM_VDP_CONTROL)
        move.w  #(PALETTE_SIZE_WORD_ENTRIES-1),d1
palette_write_palette_next_entry:
        move.w  (a0)+,(MM_VDP_DATA)
        dbf     d1,palette_write_palette_next_entry
        
        movem.l (a7)+,d0/d1/a0
        rts
        
; palette_fade_struct_update:
; For each of the palette lines, perform
; the automated fade if the fade/speed fields are set.
; Invalidates all registers.
palette_fade_struct_update:
        movem.l d0-d7/a0-a6,-(a7)
        
        move.w  #(4-1),d0                                                             ; d0 = loop counter, looping palette lines 3-0
        lea     palette_fade_struct+(palette_fade_struct_size_bytes*(4-1)),a2         ; Point to the last palette line and go backwards following d0!
        
palette_fade_struct_update_next_line:
        move.l  palette_fade_struct_fade_speed(a2),d1
        beq     palette_fade_struct_update_fade_idle
        
        add.l   d1,palette_fade_struct_fade_value(a2)
        move.l  palette_fade_struct_fade_value(a2),d1
        
        ; If value >= 256, move ptr b into ptr a and set the fade and speed to zero.
        cmp.l   #256,d1
        blt     palette_fade_struct_update_fade_do_not_cap
        move.l  #0,d1
        move.l  d1,palette_fade_struct_fade_value(a2)
        move.l  d1,palette_fade_struct_fade_speed(a2)
        move.l  palette_fade_struct_ptr_b(a2),palette_fade_struct_ptr_a(a2)
palette_fade_struct_update_fade_do_not_cap:        
        move.l  palette_fade_struct_ptr_a(a2),a0
        move.l  palette_fade_struct_ptr_b(a2),a1
        jsr     palette_write_interpolated_palette
        
palette_fade_struct_update_fade_idle:        
        lea     -palette_fade_struct_size_bytes(a2),a2
        dbf     d0,palette_fade_struct_update_next_line
        
        movem.l (a7)+,d0-d7/a0-a6
        rts

; plane_b_scrolling_background_update:
; Call once a frame to scroll background in Plane B using current state.        
; No registers invalidated.
plane_b_scrolling_background_update:
        movem.l d0,-(a7)
        
        ; Advance and loop scroll counter. Fixed point .2, max value <512.
        move.l  plane_b_scrolling_background_status_struct_scroll_speed,d0
        add.l   d0,plane_b_scrolling_background_status_struct_scroll_counter
        and.l   #((512<<2)-1),plane_b_scrolling_background_status_struct_scroll_counter
        ; Scroll Plane B according to this value: write to VSRAM and Horizontal Scroll ram.
        ; Also don't forget that the scrolling values are the pixel in Plane-space shown in the top left.
        move.l  plane_b_scrolling_background_status_struct_scroll_counter,d0
        lsr.l   #2,d0
        vram_write (VDP_LCOMMAND_VRAM_WRITE),$F400+2
        move.w  d0,(MM_VDP_DATA)
        vram_write (VDP_LCOMMAND_VSRAM_WRITE),$0000+2
        move.w  d0,(MM_VDP_DATA)
                      
        movem.l (a7)+,d0
        rts        
        
; string_length:        
; Given a 0-terminated string in a0, return
; string length in d0.l.
string_length:
        movem.l a0,-(a7)
        
        move.l  #0,d0
string_length_test_next_char:
        cmpi.b  #0,(a0)+
        beq     string_length_end
        add.l   #1,d0
        jmp     string_length_test_next_char
        
string_length_end:
        movem.l (a7)+,a0
        rts
        
; draw_string_centred_a1:
; d0.l: plane (0=A, 1=W, 2=B)
; d1.l: X center
; d2.l: Y
; d3.l: palette line
; d4.l: width of plane to write into for clipping = 64
; a0  : pointer to null-terminated string
; a1  : pointer to which string drawing function to use
; No registers invalidated.
draw_string_centred_a1:    
        ; Subtract half string length from d1, saving d0 temporarily.
        movem.l d1,-(a7)
        movem.l d0,-(a7)
        jsr     string_length
        lsr.l   #1,d0
        sub.l   d0,d1
        movem.l (a7)+,d0
        
        jsr     (a1)
        movem.l (a7)+,d1
        rts
        
; draw_string_8x8:
; Assumes 8x8 font is at $B420 (tile 1441) and tile 0 is transparent.
; d0.l: plane (0=A, 1=W, 2=B)
; d1.l: X
; d2.l: Y
; d3.l: palette line
; d4.l: width of plane to write into for clipping = 64
; a0  : pointer to null-terminated string
; No registers invalidated.
draw_string_8x8:
        movem.l d3,-(a7)
        
        lsl.w   #8,d3
        lsl.w   #5,d3                                                                 ; d3 = Palette line in bit 13 for tegel construction
        add.w   #1441,d3                                                              ; d3 = Tegel for '!', we can add character code onto this
        
        jsr     draw_string_single_row
        
        movem.l (a7)+,d3
        rts
        
; draw_string_8x16:
; Assumes 8x16 font is at $9CA0 (tile 1253 top line, tile 1253+94 bottom line) and tile 0 is transparent.
; d0.l: plane (0=A, 1=W, 2=B)
; d1.l: X
; d2.l: Y
; d3.l: palette line
; d4.l: width of plane to write into for clipping = 64
; a0  : pointer to null-terminated string
; No registers invalidated.
draw_string_8x16:
        movem.l d2/d3,-(a7)
        
        lsl.w   #8,d3
        lsl.w   #5,d3                                                                 ; d3 = Palette line in bit 13 for tegel construction
        add.w   #1253,d3                                                              ; d3 = Tegel for '!', we can add character code onto this
        
        ; Call a function twice to write the top and bottom halves of the string, 
        ; advancing the base tegel number and the Y coordinate between the two.
        jsr     draw_string_single_row        
        add.w   #1,d2
        add.w   #94,d3        
        jsr     draw_string_single_row
        
        movem.l (a7)+,d2/d3
        rts
        
; draw_string_single_row:     
; Do not use directly.
; Helper function used by draw_string_8x8 and draw_string_8x16
; Draws a horizontal row of tiles from a single tegel source,
; which can be the 8x8 font or one half of the 8x16 font.
; d0.l: plane (0=A, 1=W, 2=B)
; d1.l: X
; d2.l: Y
; d3.l: base tegel number including palette line
; d4.l: width of plane to write into for clipping = 64
; a0  : pointer to null-terminated string
; No registers invalidated.
draw_string_single_row:        
        movem.l d0-d7/a0,-(a7)
        
        ; Calculate VRAM address based on plane and coordinate.
        ; $C000 + (plane * $1000) + 2*(X + Y*64) 
        lsl.w   #6,d2                                                                 ; d2 = (Y * 64)
        or.w    d1,d2                                                                 ; d2 = (Y * 64 + X)
        lsl.w   #1,d2                                                                 ; d2 = (Y * 64 + X) * 2
        lsl.w   #8,d0
        lsl.w   #4,d0                                                                 ; d0 = ($1000 * plane)
        or.w    #$C000,d0                                                             ; d0 = ($1000 * plane) + $C000
        or.w    d2,d0                                                                 ; d0 = ($1000 * plane) + $C000 + (Y * 64 + X) * 2
        jsr     vram_write_swizzle_address_in_d0                                      ; Command complete        
        move.l  d0,(MM_VDP_CONTROL)                                                   ; VDP set up to write to address
                
        ; Calculate how many cells we are allowed to write,
        ; also subtract 1 for loop dbf compatibility.
        sub.w   #1,d4
        sub.w   d1,d4                                                                 ; d4 = (planewidth-1)-X
        
        ; Tegel = 1441 + (character-'!') | palette line in d3.
        ; If character==' ' then write tegel zero.
        ; If character==0 then abort.
draw_string_single_row_next_character:
        move.w  #0,d0                                                                ; Clear d0 for character loading
        move.b  (a0)+,d0                                                             ; d0 = character
        beq     draw_string_single_row_complete                                      ; Abort if character = 0
        sub.w   #'!',d0                                                              ; Generate exclamation-mark-based tile number
        bge     draw_string_single_row_next_character_exclamation_or_better          ; Branch if above space (>='!') or invisible (<'!')
        move.w  #0,d0                                                                ; Write blank tegel if <'!'
        jmp     draw_string_single_row_next_character_write_d0_tegel   
draw_string_single_row_next_character_exclamation_or_better:
        cmp.w   #('~'-'!'),d0                                                        ; Notice that exclation mark is zero now.
        ble     draw_string_single_row_next_character_is_printable                   ; Branch if tilde or below (<='~') or unprintable (>'~')
        move.w  #0,d0                                                                ; Write blank tegel if >'~'
        jmp     draw_string_single_row_next_character_write_d0_tegel   
draw_string_single_row_next_character_is_printable:         
        add.w   d3,d0                                                                ; If >='!', construct tegel in d0
draw_string_single_row_next_character_write_d0_tegel:        
        move.w  d0,(MM_VDP_DATA)                                                     ; Write current tegel
        dbf     d4,draw_string_single_row_next_character                             ; Next
        
draw_string_single_row_complete:
        movem.l (a7)+,d0-d7/a0
        rts
        
; vram_write_swizzle_address_in_d0:
; Given an address in d0.w, alter it to a longword VDP command
; to write to the given VRAM address.
; Doesn't actually send it, in case you want to 
; do some maths on it.
; Only alters d0.        
vram_write_swizzle_address_in_d0:
        movem.l d1,-(a7)
        
        move.w  d0,d1
        and.w   #%0011111111111111,d0                                                 ; d0 = Lowest 14 bits of address
        swap    d0                                                                    ; d0 = Lowest 14 bits of address in bit 16 upwards
        lsr.w   #8,d1
        lsr.w   #6,d1                                                                 ; d1 = Highest 2 bits of address in bit 0
        or.w    d1,d0                                                                 ; d0 = VDP command compatible address
        or.l    #VDP_LCOMMAND_VRAM_WRITE,d0                                           ; d0 = VDP command complete
        
        movem.l (a7)+,d1
        rts       
                
        align   4       
intro_screen_handshake_response_string_table:
        dc.l    intro_screen_handshake_response_string_ok
        dc.l    intro_screen_handshake_response_string_sd_card_error
        dc.l    intro_screen_handshake_response_string_no_contents
        dc.l    intro_screen_handshake_response_string_unknown
        
string_unused:
        dc.b    '(c) 2020 Mathew Carr',0  
string_intro_game_raccoon_game_development_cartridge:
        dc.b    'Game Development Cartridge',0      
string_intro_handshaking:
        dc.b    'Handshaking...',0
intro_screen_handshake_response_string_ok:
        dc.b    'Ready!',0        
intro_screen_handshake_response_string_sd_card_error:
        dc.b    'SD Card Error',0        
intro_screen_handshake_response_string_no_contents:
        dc.b    'No Files Found',0        
intro_screen_handshake_response_string_unknown:
        dc.b    'Error Loading Card',0    
string_boot:
        dc.b    'BOOT',0
string_browse:
        dc.b    'BROWSE',0
string_about:
        dc.b    'ABOUT',0
string_select:
        dc.b    'SELECT',0
string_parent:
        dc.b    'PARENT',0
string_directory_is_empty:
        dc.b    '( empty )',0
string_regeneration_in_progress:
        dc.b    'Scanning SD Card',0     
string_regeneration_complete:        
        dc.b    'Directory scan complete',0 
string_regeneration_failed:       
        dc.b    'Directory scan failed',0  
string_regeneration_failed_error:       
        dc.b    'A directory error occured',0  
string_regeneration_failed_string_too_long:       
        dc.b    'Pathnames are too long',0  
string_regeneration_failed_limit_reached:       
        dc.b    'Too many files on SD Card',0  
string_installation_please_restart_your_system:
        dc.b    'Please restart your system',0
string_installation_in_progress:
        dc.b    'Installation in progress',0
string_installation_complete:        
        dc.b    'Installation complete',0
string_installation_failed:       
        dc.b    'Installation failed',0
string_installation_failure_file_too_big:       
        dc.b    'Selected file is too big',0
string_installation_failure_read_error:       
        dc.b    'Error reading from SD Card',0
string_installation_failure_verify_error:       
        dc.b    'Error writing to SD Card',0

        align   4             
        
ramresident_palettes = *        
        incbin  "palettes.bin"

palette_black                          = ramresident_palettes + (PALETTE_SIZE_BYTES* 0)
palette_game_raccoon_logo_blue_font    = ramresident_palettes + (PALETTE_SIZE_BYTES* 1)
palette_game_raccoon_logo_yellow_font  = ramresident_palettes + (PALETTE_SIZE_BYTES* 2)
palette_plane_b_red_red                = ramresident_palettes + (PALETTE_SIZE_BYTES* 3)
palette_plane_b_darkblue_darkblue      = ramresident_palettes + (PALETTE_SIZE_BYTES* 4)
palette_plane_b_lightblue_lightblue    = ramresident_palettes + (PALETTE_SIZE_BYTES* 5)
palette_solid_lightblue_transition     = ramresident_palettes + (PALETTE_SIZE_BYTES* 6)
palette_plane_b_lightblue_darkblue     = ramresident_palettes + (PALETTE_SIZE_BYTES* 7)
        
        align   4
        
        include "controller_state.68k.asm"
        
        align   4             

game_raccoon_ram_resident_assembled_block_end = *        
 
GAME_RACCOON_RAM_RESIDENT_ROM_BLOCK_LENGTH_BYTES = game_raccoon_ram_resident_assembled_block_end-game_raccoon_ram_resident_assembled_block_begin
GAME_RACCOON_RAM_RESIDENT_ROM_BLOCK_LENGTH_LONGS = (GAME_RACCOON_RAM_RESIDENT_ROM_BLOCK_LENGTH_BYTES+3/4)
               
; End of Game Raccoon RAM Resident part.
; The above has been assembled with * = $FF0000 onwards,
; but is located in the image contiguous to the entry point,
; at aroundabouts $002000 onwards.
; We measure the size of the generated block in longwords and
; use this to copy the assembled code to RAM for real in the
; bootstrap so we can jump to the $FF0000-based labels properly.
               
; --------------------------------------------------------------------------------
               
; Variables are present in RAM beyond the installed RAM-resident block.
; This means we need to declare all the global variables that both
; the RAM-resident and non-RAM-resident parts will use together.
; We can use all the ram from asterisk up to however large we like
; as long as we don't crash into the stack.
; Vasm assembler is smart enough to convert these labels into
; PC-relative addressing for shorter LEA instructions and so on.
; Asmx isn't so clever, but we're not angling for speed, so ignore it.
; The only thing we need to do is make sure that all of these labels
; are consistent and monotonically increasing. 

        align   4

; Longword used to store number of frames until released from
; isolation. Used for simulating action of Game Raccoon cartridge
; in emulators.
simulated_isolation_timer = *
        ds      4        
        
pad_state                        = *
        ds      PAD_STATUS_TOTAL_SIZE
pad_state_end                    = *

plane_b_scrolling_background_status_struct_scroll_counter      = *
        ds      4
plane_b_scrolling_background_status_struct_scroll_speed        = *
        ds      4
        
; There is a copy of this struct for each palette line (four lines)
; to handle asynchronous fading/loading.        
palette_fade_struct            = *
        
palette_fade_struct_ptr_a      = (0*4)
palette_fade_struct_ptr_b      = (1*4)      
palette_fade_struct_fade_value = (2*4)      
palette_fade_struct_fade_speed = (3*4) ; 0 static, != 0 fading up to 255, then stopping.
     
palette_fade_struct_size_bytes = (4*4)
        ds      (4*palette_fade_struct_size_bytes)
        
palette_fade_struct_size_all_lines_words = ((4*palette_fade_struct_size_bytes)/2)
        
palette_fade_0_ptr_a      = palette_fade_struct+(palette_fade_struct_size_bytes*0)+palette_fade_struct_ptr_a
palette_fade_0_ptr_b      = palette_fade_struct+(palette_fade_struct_size_bytes*0)+palette_fade_struct_ptr_b
palette_fade_0_fade_value = palette_fade_struct+(palette_fade_struct_size_bytes*0)+palette_fade_struct_fade_value
palette_fade_0_fade_speed = palette_fade_struct+(palette_fade_struct_size_bytes*0)+palette_fade_struct_fade_speed
palette_fade_1_ptr_a      = palette_fade_struct+(palette_fade_struct_size_bytes*1)+palette_fade_struct_ptr_a
palette_fade_1_ptr_b      = palette_fade_struct+(palette_fade_struct_size_bytes*1)+palette_fade_struct_ptr_b
palette_fade_1_fade_value = palette_fade_struct+(palette_fade_struct_size_bytes*1)+palette_fade_struct_fade_value
palette_fade_1_fade_speed = palette_fade_struct+(palette_fade_struct_size_bytes*1)+palette_fade_struct_fade_speed
palette_fade_2_ptr_a      = palette_fade_struct+(palette_fade_struct_size_bytes*2)+palette_fade_struct_ptr_a
palette_fade_2_ptr_b      = palette_fade_struct+(palette_fade_struct_size_bytes*2)+palette_fade_struct_ptr_b
palette_fade_2_fade_value = palette_fade_struct+(palette_fade_struct_size_bytes*2)+palette_fade_struct_fade_value
palette_fade_2_fade_speed = palette_fade_struct+(palette_fade_struct_size_bytes*2)+palette_fade_struct_fade_speed
palette_fade_3_ptr_a      = palette_fade_struct+(palette_fade_struct_size_bytes*3)+palette_fade_struct_ptr_a
palette_fade_3_ptr_b      = palette_fade_struct+(palette_fade_struct_size_bytes*3)+palette_fade_struct_ptr_b
palette_fade_3_fade_value = palette_fade_struct+(palette_fade_struct_size_bytes*3)+palette_fade_struct_fade_value
palette_fade_3_fade_speed = palette_fade_struct+(palette_fade_struct_size_bytes*3)+palette_fade_struct_fade_speed
        
; Screen state variables:

; INTRO SCREEN:
intro_screen_state = *
        ds      4        
intro_screen_state_choices_available = *
        ds      4
        
; ABOUT SCREEN:
about_screen_state = *
        ds      4        
about_screen_scroll_current_position_pixels = *
        ds      4
about_screen_scroll_maximum_position_pixels = *
        ds      4
about_screen_start_hold_counter = *
        ds      4

; BROWSE SCREEN:
browse_screen_fade_state = *
        ds      4
        
browse_screen_currently_displayed_directory_record_cache_ptr    = *    
        ds      4    
            
browse_screen_selected_item_in_list_index                       = *
        ds      4    
        
browse_screen_cursor_current_position_pixels                    = *
        ds      4
browse_screen_cursor_destination_position_pixels                = *
        ds      4        
        
browse_screen_scroll_current_position_pixels                    = *
        ds      4
browse_screen_scroll_destination_position_pixels                = *
        ds      4        
        
; 0 for empty directory.        
browse_screen_no_items                                          = *
        ds      4        
browse_screen_no_directories                                    = *
        ds      4
browse_screen_no_files                                          = *
        ds      4
        
; The directory cache items list contains the items can be shown on
; any single page in the interface.
; They are longword sized, since they are pointers (to directory entries in ROM).
DIRECTORY_CACHE_ITEMS_LIST_SIZE_IN_ITEMS = 1024
directory_cache_items_list_begin = *
        ds      (4*DIRECTORY_CACHE_ITEMS_LIST_SIZE_IN_ITEMS)
directory_cache_items_list_end   = *        
        
        
; FLASH WARNING SCREEN:
flash_warning_screen_state = *
        ds      4        
flash_warning_screen_buttons_hold_counter = *
        ds      4
        
; REGENERATION SCREEN:
regeneration_screen_fade_state = *
        ds      4
regeneration_screen_raccoon_animation_state = *
        ds      4                           ; Placeholder variable for something to do with the animation.
regeneration_screen_raccoon_animation_frame = *
        ds      4                           ; Placeholder variable for something to do with the animation.
regeneration_screen_isolation_in_effect = *
        ds      4     
regeneration_screen_response_word_from_pic:
        ds      4                           ; Write a word into this address, ignore the unused two following bytes.      
        
; INSTALLATION SCREEN:
installation_screen_fade_state = *
        ds      4        
; These fields below are the interface to the screen.        
installation_screen_chosen_word = *
        ds      4                           ; Write a word into this address, ignore the unused two following bytes.
installation_screen_should_show_boot_on_success = *
        ds      4                           ; This is false if we've written the menu firmware.
installation_screen_file_size_in_sectors = *
        ds      4                           ; Used as a heuristic for the progress bar. This is definitely a long.
; These fields are internal to the screen.
installation_screen_expected_frames = *
        ds      4                           ; Number of frames expected installation will last based on file size.
installation_screen_elapsed_frames = *
        ds      4                           ; Counts number of frames elapsed on installation screen. Capped at sectors * 278        
installation_screen_isolation_in_effect = *
        ds      4        
installation_screen_raccoon_animation_state = *
        ds      4                           ; Placeholder variable for something to do with the animation.
installation_screen_raccoon_animation_frame = *
        ds      4                           ; Placeholder variable for something to do with the animation.
installation_screen_response_word_from_pic:
        ds      4                           ; Write a word into this address, ignore the unused two following bytes.      
installation_screen_choices_available:
        ds      4        
        